//  STEFit
//   Fit STE spectra to known door source lines
//
//  D.W.Curtis 2004-5-19


#include <userint.h>
#include "Decom.h"
#include <ansi_c.h>

extern int PlotSTESpecPanel;
static int STEFitPanel=-1;
unsigned int GetWord(int i);
int GetByte(int i);
long GetLong3412(int i);
int GetApID(void);
int PacketNumber;
float lsq(void);
extern double Spectra[];
extern int AccTime; 
extern int NumEnergy0;

extern float Y[], Sig[];
extern int nY,nP;
extern int nF, P0[];
extern float P1[];
extern float (*func[10])(float x, float P[]), (*dfunc[10])(int iX, int iP, float P[]);

#define REGIONS 3
#define PEAKS 4
#define ENERGYGAIN (REGIONS+2*PEAKS)
#define ENERGYOFFSET (REGIONS+2*PEAKS+1)

static int LowerLimit[REGIONS] = { 10, 45, 216 };
static int UpperLimit[REGIONS] = { 20, 75, 240 };
static int RegionLimits[REGIONS+1];
static float Energy[PEAKS] = { 5.9F, 22.1F, 25.0F, 88.04F };
static int PeakRegion[PEAKS] = { 0, 1, 1, 2 };
static int Scale;

extern char *Result;

static float gaus(float x, float bin, float amp, float sig)	/* compute gaussianlocated at bin, amplitude amp */
															/* sigma sig, for value x */
{
	if((x = (x-bin)/sig) > 5.) return(0.F);
	if(x < -5.) return(0.F);

	return((float) (amp*exp((double)-x*x/2.F)));
}


// Fit parameters:
//  i=0 - REGIONS-1 = baseline for region i
//  i=REGIONS - REGIONS+PEAKS-1 = amplitude for peak i-REGIONS
//  i=REGIONS+PEAKS - REGIONS+2*PEAKS-1 = sigma for peak i-REGIONS-PEAKS
//  i=ENERGYGAIN = energy gain
//  i=ENERGYOFFSET = energy offset
static float Fit(float x, float PP[])	/* compute fit value */
{
	float F, x1, bin;            
	int i, Region;

	for(i=0; i<REGIONS; i++) if((x >= RegionLimits[i]) && (x < RegionLimits[i+1])) Region = i;
	F=PP[Region];
	for(i=0; i<PEAKS; i++) {
		if(PeakRegion[i] == Region) {
			bin = (Energy[i]-PP[ENERGYOFFSET])/PP[ENERGYGAIN] - (LowerLimit[Region]/Scale)+RegionLimits[Region];
			F += gaus(x, bin, PP[REGIONS+i], PP[REGIONS+PEAKS+i]);
		}
	}
	return F;
}


static float dFit(int iX, int iP, float PP[])
{
	int i, Region;
	float dF, bin;
	
	for(i=Region=0; i<REGIONS; i++) if((iX >= RegionLimits[i]) && (iX < RegionLimits[i+1])) Region = i;
	if(iP < REGIONS) {	// baseline
		if(iP == Region) return 1.F;
		else return 0.F;
	}
	
	if(iP < REGIONS+PEAKS) {	//amplitude
		if(PeakRegion[iP-REGIONS] != Region) return 0.F;
		bin =  (Energy[iP-REGIONS]-PP[ENERGYOFFSET])/PP[ENERGYGAIN]- (LowerLimit[Region]/Scale)+RegionLimits[Region];
		return gaus((float)iX, bin, 1.F, PP[iP+PEAKS]);
	}
	
	if(iP < REGIONS+2*PEAKS) {	// sigma
		if(PeakRegion[iP-REGIONS-PEAKS] != Region) return 0.F;
		bin =  (Energy[iP-REGIONS-PEAKS]-PP[ENERGYOFFSET])/PP[ENERGYGAIN]- (LowerLimit[Region]/Scale)+RegionLimits[Region];
		return gaus((float)iX, bin, PP[iP-PEAKS], PP[iP]) * ((float)iX-bin)*((float)iX-bin)/(PP[iP]*PP[iP]*PP[iP]);
	}
		
	if(iP == ENERGYGAIN) {	// energy gain
		for(i=0, dF=0.F; i<PEAKS; i++) if(PeakRegion[i] == Region) {
			bin =  (Energy[i]-PP[ENERGYOFFSET])/PP[ENERGYGAIN]- (LowerLimit[Region]/Scale)+RegionLimits[Region];
			dF -= gaus((float)iX, bin, PP[REGIONS+i], PP[REGIONS+PEAKS+i])
				* ((float)iX-bin) * (Energy[i]-PP[ENERGYOFFSET]) / 
				(PP[REGIONS+PEAKS+i]*PP[REGIONS+PEAKS+i]*PP[ENERGYGAIN]*PP[ENERGYGAIN]);
		}
		return dF;
	}
	
	if(iP == ENERGYOFFSET) {	//energy offset
		for(i=0, dF=0.F; i<PEAKS; i++) if(PeakRegion[i] == Region) {
			bin =  (Energy[i]-PP[ENERGYOFFSET])/PP[ENERGYGAIN]- (LowerLimit[Region]/Scale)+RegionLimits[Region];
			dF -= gaus((float)iX, bin, PP[REGIONS+i], PP[REGIONS+PEAKS+i])
				* ((float)iX-bin) / 
				(PP[REGIONS+PEAKS+i]*PP[REGIONS+PEAKS+i]*PP[ENERGYGAIN]);
		}
	}
	return dF;
}

#define SIG2FWHM	2.35482F	/* scale factor to convert sigma to FWHM */
#define G   2.5066F				/* scale factor from Amplitude*Sigma to integrated counts */

static void FitSTE()
{
	int i, j, k, RATES, SELECT, SINGLE, DETECTOR, j0;
	float Chisq;
	float X[256];
	static float Rate0[PEAKS] = { 15.F, 10.F, 2.F, .003F };
	static float Base0[REGIONS] = {  2.F, 1.F,  0.001F };
	
	nP = REGIONS+2*PEAKS+2;
	nF = 1;
	P0[0] = 0;
	func[0] = Fit;
	dfunc[0] = dFit;
	
	GetCtrlVal(STEFitPanel, STEFIT_Select, &SELECT);
	j0 = SELECT?(NumEnergy0*4):0;
	GetCtrlVal(PlotSTESpecPanel, STESPEC_SingleDet, &SINGLE);
	GetCtrlVal(STEFitPanel, STEFIT_Detector, &DETECTOR);
	if(!SINGLE) j0 += DETECTOR*NumEnergy0;
	
	Scale = 256/NumEnergy0;
	if(SINGLE) Scale /= 4;

	RegionLimits[0] = 0;
	for(i=nY=0; i<REGIONS; i++) {
		for(j=(LowerLimit[i]/Scale),k=0; j<=((UpperLimit[i]+Scale-1)/Scale); j++,k++) Y[nY++]=Spectra[j+j0];
		RegionLimits[i+1] = RegionLimits[i]+k;
	}
	
	for(i=0; i<nY; i++) Sig[i] = sqrt(Y[i])+1.F;
	
	Result = "OK";
	
	for(i=0; i<REGIONS; i++) P1[i] = Base0[i]*AccTime*Scale;	// baseline
	for(i=0; i<PEAKS; i++) P1[REGIONS+i] = Rate0[i]*AccTime*Scale;	// amplitude
	for(i=0; i<PEAKS; i++) P1[REGIONS+PEAKS+i] = 2.F/Scale;	//sigma
	P1[ENERGYGAIN] = 100.F*Scale/256.F;
	P1[ENERGYOFFSET] = 0.F;

	Chisq = lsq();

	SetCtrlVal(STEFitPanel, STEFIT_ChiSquared, Chisq);
	ResetTextBox(STEFitPanel, STEFIT_Msg, Result);
	SetCtrlVal(STEFitPanel, STEFIT_Baseline0, P1[0]/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_Baseline1, P1[1]/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_Baseline2, P1[2]/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude0, P1[3]*P1[7]*G/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude1, P1[4]*P1[8]*G/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude2, P1[5]*P1[9]*G/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude3, P1[6]*P1[10]*G/AccTime);
	SetCtrlVal(STEFitPanel, STEFIT_FWHM0, P1[7]*SIG2FWHM*P1[ENERGYGAIN]);
	SetCtrlVal(STEFitPanel, STEFIT_FWHM1, P1[8]*SIG2FWHM*P1[ENERGYGAIN]);
	SetCtrlVal(STEFitPanel, STEFIT_FWHM2, P1[9]*SIG2FWHM*P1[ENERGYGAIN]);
	SetCtrlVal(STEFitPanel, STEFIT_FWHM3, P1[10]*SIG2FWHM*P1[ENERGYGAIN]);
	SetCtrlVal(STEFitPanel, STEFIT_Gain, P1[ENERGYGAIN]);
	SetCtrlVal(STEFitPanel, STEFIT_Offset, P1[ENERGYOFFSET]);
	
	GetCtrlVal(PlotSTESpecPanel, STESPEC_RATES, &RATES);

	for(j=0; j<REGIONS; j++) {
		for(nY=0,i=RegionLimits[j]; i<RegionLimits[j+1]; i++) {
			Y[nY] = Fit((float)i, P1);
			if(RATES) Y[nY] /= AccTime;
			X[nY++] = (float)(i-RegionLimits[j]+LowerLimit[j]/Scale);
		}
		PlotXY (PlotSTESpecPanel, STESPEC_GRAPH, X, Y, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
    }
	
}


int CVICALLBACK STEFit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			if(STEFitPanel == -1) STEFitPanel = LoadPanel(PlotSTESpecPanel, "DECOM.uir", STEFIT);
			DisplayPanel(STEFitPanel);
			FitSTE();
			break;
		}
	return 0;
}


int CVICALLBACK HideSTEFit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
		}
	return 0;
}
