// Decom.c
//
//  STEREO IMPACT Project
//  Telemetry file decomutator
//
//  David Curtis  2002-11-14
//

#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include <formatio.h>
#include "DECOM.h"

char FileName[500] = "log";
FILE *DumpFile=NULL;
int MainPanel, ApIDPanel, PacketPanel, HKPPanel, TimingPanel, MagPanel, FHKPPanel;
int ScanHKPPanel, ScanBurstPanel, PlotBurstPanel, PlotHKPPanel, PlotSEPHKPPanel;
int PlotSTERatesPanel, PlotSTESpecPanel, PlotMAGPanel, PlotSWEARatesPanel, PlotBurstCritPanel;
int PlotPktPanel;
unsigned char Data[298];

extern struct Database {
	char ID[30];
	int Length;
	int Offset;
	char Type[10];
	int Width;
	double A1;
	double A0;
	char Units[10];
	double RL;
	double YL;
	double YH;
	double RH;
} Database[300];
extern int NDB;
void init_database(void);

int main (int Argc, char *argv[])
{
	int File, n;

	if (InitCVIRTE (0, argv, 0) == 0) return -1;
	
	MainPanel = LoadPanel(0, "DECOM.uir", DECOM);
	ApIDPanel = LoadPanel(0, "DECOM.uir", ApID);
	PacketPanel = LoadPanel(0, "DECOM.uir", PACKET);
	HKPPanel = LoadPanel(0, "DECOM.uir", HKP);
	TimingPanel = LoadPanel(0, "DECOM.uir", TIMING);
	MagPanel = LoadPanel(0, "DECOM.uir", MAG);
	FHKPPanel = LoadPanel(0, "DECOM.uir", SWEAFHKP); 
	ScanHKPPanel = LoadPanel(0, "DECOM.uir", SCANHKP);
	ScanBurstPanel = LoadPanel(0, "DECOM.uir", SCANBURST);
	PlotBurstPanel = LoadPanel(0, "DECOM.uir", PLOTBURST);
	PlotHKPPanel = LoadPanel(0, "DECOM.uir", PLOTHKP);
	PlotSEPHKPPanel = LoadPanel(0, "DECOM.uir", PLOTSEPHKP);
	PlotSTERatesPanel = LoadPanel(0, "DECOM.uir", STERATES);
	PlotSTESpecPanel = LoadPanel(0, "DECOM.uir", STESPEC);
	PlotMAGPanel = LoadPanel(0, "DECOM.uir", PLOTMAG);
	PlotSWEARatesPanel = LoadPanel(0, "DECOM.uir", SWEARATES);
	PlotBurstCritPanel = LoadPanel(0, "DECOM.uir", BURSTCRIT);
	PlotPktPanel = LoadPanel(0, "DECOM.uir", PLOTPKT);
	DisplayPanel ( MainPanel );
	init_database();
	
	RunUserInterface ();
	
	return 0;
}

int CVICALLBACK Panel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_CLOSE:
			exit(0);

			break;
		}
	return 0;
}

int CVICALLBACK SelectFile (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			FileSelectPopup (FileName,  "*.tlm",  "*.tlm;*.log", "Select File to Decomutate",
				VAL_SELECT_BUTTON, 0, 0, 1, 0, FileName);
			SetCtrlVal(MainPanel, DECOM_FileName, FileName);
			SetCtrlAttribute(MainPanel, DECOM_ScanApIDs, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_DisplayPackets, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_DisplayHKP, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_DisplayTiming, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotMagPacket, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotSWEAFHKP, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_ScanHKP, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_ScanBurst, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotBurst, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotHKP, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotSEPHKP, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotSTERates, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_AccSTESpec, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_STEFit, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotMAG, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotSWEARates, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotBurstCriteria, ATTR_DIMMED, 0);
			SetCtrlAttribute(MainPanel, DECOM_PlotPkt, ATTR_DIMMED, 0);
			HidePanel(ApIDPanel);
			HidePanel(PacketPanel);
			HidePanel(HKPPanel);
			HidePanel(TimingPanel);
			HidePanel(MagPanel);
			HidePanel(ScanHKPPanel);
			HidePanel(ScanBurstPanel);
			HidePanel(PlotBurstPanel);
			HidePanel(PlotHKPPanel);
			HidePanel(PlotSEPHKPPanel);
			HidePanel(PlotSTERatesPanel);
			HidePanel(PlotSTESpecPanel);
			HidePanel(PlotMAGPanel);
			HidePanel(PlotSWEARatesPanel);
			HidePanel(PlotBurstCritPanel);
			HidePanel(PlotPktPanel);
			if(DumpFile) fclose(DumpFile);
			DumpFile = fopen(FileName, "rb");
			break;
		}
	return 0;
}

int CVICALLBACK GenericPanel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_CLOSE:
			HidePanel(panel);
			break;
		}
	return 0;
}

void SwapTelemetryBytes(char *Data)
{
	int i,c, Swap;
	
	GetCtrlVal(MainPanel, DECOM_ByteSwap, &Swap);
	if(!Swap) return;
	for(i=26; i<298; i+=2) {
		c=Data[i];
		Data[i] = Data[i+1];
		Data[i+1] = c;
	}
}

static int SwapApID(unsigned char *Data) 
{
	int Swap;
	
	GetCtrlVal(MainPanel, DECOM_ByteSwap, &Swap);
	if(Swap) return Data[27] + ((Data[26]&7)<<8);
	return Data[26] + ((Data[27]&7)<<8);
}

static int SwapSequence(unsigned char *Data) 
{
	int Swap;
	
	GetCtrlVal(MainPanel, DECOM_ByteSwap, &Swap);
	if(Swap) return Data[29] + ((Data[28]&0x3F)<<8);
	return Data[28] + ((Data[29]&0x3F)<<8);
}

int SwapWord(unsigned char *Data, int X) 
{
	int Swap;
	
	GetCtrlVal(MainPanel, DECOM_ByteSwap, &Swap);
	if(Swap) return Data[27+X*2] + (Data[26+X*2]<<8);
	return Data[26+X*2] + (Data[27+X*2]<<8);
}

static int ChecksumPacket(char *Data)
{
	int i, Sum;

	for(i=Sum=0; i<272; i++)
		Sum += Data[26+i];
		
	return Sum&0xFF;
}

static int LegalApID(int ID)
{
	if(ID < 0x100) return 1;		// Spacecraft
	if((ID < 0x200) || (ID > 0x37F)) return 0;
	if((ID > 0x27F) && (ID < 0x300)) return 0;
	return 1;
}

int GetByte(int i)
{
	return Data[i];
}

static int GetSwappedByte(int i) 
{
	int Swap;
	
	GetCtrlVal(MainPanel, DECOM_ByteSwap, &Swap);
	if(Swap) return Data[i^1];
	return Data[i];
}

unsigned int GetWord(int i)
{
	return Data[i] + (Data[i+1]<<8);
}

unsigned int GetWord21(int i)
{
	return Data[i+1] + (Data[i]<<8);
}

long GetLong3412(int i)
{
	return (GetWord(i)<<16) + GetWord(i+2);
}

long GetLong4321(int i)
{
	return GetWord21(i+2) + (GetWord21(i)<<16);
}

long GetLong(int i)
{
	return GetWord(i) + (GetWord(i+2)<<16);
}


long FIFTYEIGHT=1830297600;

char *GetTime()
{
	long T;
	static char dtime[45],atime[26],fsec[20];

	T = GetLong3412(32)+FIFTYEIGHT;
	strcpy(atime,asctime(gmtime(&T)));
	atime[5]-=0x20;		//Captialize
	atime[6]-=0x20;		//Captialize
	memcpy(dtime,atime+20,4); //YYYY
	memcpy(dtime+4,atime+3,4); // MMM
	memcpy(dtime+8,atime+7,3); // DD
	memcpy(dtime+11,atime+10,9); // HH:MM:SS
	sprintf(fsec,"%3.2f",GetByte(37)/256.);
	memcpy(dtime+20,fsec+1,7);
	dtime[27]='\0';
	return dtime;
}

char *GetTimeL()
{
	long T;
	static char dtime[45],atime[26],fsec[20];

	T = GetLong3412(32)+FIFTYEIGHT;
	strcpy(atime,asctime(gmtime(&T)));
	atime[5]-=0x20;		//Captialize
	atime[6]-=0x20;		//Captialize
	memcpy(dtime,atime+20,4); //YYYY
	memcpy(dtime+4,atime+3,4); // MMM
	memcpy(dtime+8,atime+7,3); // DD
	memcpy(dtime+11,atime+10,9); // HH:MM:SS
	sprintf(fsec,"%5.4f",GetByte(37)/256.+GetByte(36)/65536.);
	memcpy(dtime+20,fsec+1,7);
	dtime[27]='\0';
	return dtime;
}

char *GetTime1(unsigned int UTCs, char *dtime)
{
	long T;
	static char atime[26];

	T = UTCs+FIFTYEIGHT;
	strcpy(atime,asctime(gmtime(&T)));
	atime[5]-=0x20;		//Captialize
	atime[6]-=0x20;		//Captialize
	memcpy(dtime,atime+20,4); //YYYY
	memcpy(dtime+4,atime+3,4); // MMM
	memcpy(dtime+8,atime+7,3); // DD
	memcpy(dtime+11,atime+10,9); // HH:MM:SS
	dtime[20]='\0';
	return dtime;
}

char *GetTime2(unsigned int UTCs, char *dtime)
{
	long T;
	static char atime[26];

	T = UTCs+FIFTYEIGHT;
	strcpy(atime,asctime(gmtime(&T)));
	atime[5]-=0x20;		//Captialize
	atime[6]-=0x20;		//Captialize
	memcpy(dtime,atime+10,9); // HH:MM:SS
	dtime[9]='\0';
	return dtime;
}

char *GetTimeX(unsigned int UTCs, char *dtime)
{
	long T;
	static char atime[26];

	T = UTCs+FIFTYEIGHT;
	strcpy(atime,asctime(gmtime(&T)));
	atime[5]-=0x20;		//Captialize
	atime[6]-=0x20;		//Captialize
	memcpy(dtime,atime+4,3); // MMM
	dtime[3] = '/';
	memcpy(dtime+4,atime+8,2); // DD
	dtime[6] = '/';
	memcpy(dtime+7,atime+20,4); //YYYY
	memcpy(dtime+11,atime+10,9); // HH:MM:SS
	dtime[20]='\0';
	return dtime;
}

unsigned long GetUTC(char *time)
{
	struct tm tm;
	char Mon[10];
	static char *Months[] = { "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
	int i, j;
	
	sscanf(time, "%d %s %d %d:%d:%d", &tm.tm_year, Mon, &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
	for(i=j=0; (i<12) && (j<3); i++) for(j=0; j<3; j++) if(Months[i][j] != Mon[j]) break;
	tm.tm_mon = i-1;
	tm.tm_isdst = 0;
	tm.tm_year -= 1900;
	
	return mktime(&tm)-FIFTYEIGHT-8*60*60;
}

int GetData()
{
	int AR, ID;
	static unsigned long UTC, UTC1, UTC2;
	static FILE *File=NULL;
	char Str[100];

	GetCtrlVal(MainPanel, DECOM_FullRange, &AR);
	if(File == NULL) {
		File = fopen(FileName, "rb");
		if(AR) {
			if(fread(Data, 298, 1, File) == 0) {
				fclose(File);
				File = NULL;
				return 0;
			}
			UTC = GetLong4321(32);
			GetTime1(UTC, Str);
			ResetTextBox(MainPanel, DECOM_UTCStart, Str);
			return 1;
		}
		GetCtrlVal(MainPanel, DECOM_UTCStart, Str);
		UTC1 = GetUTC(Str);
		GetCtrlVal(MainPanel, DECOM_UTCEnd, Str);
		UTC2 = GetUTC(Str);
		UTC=0;
		while(UTC < UTC1) {
			if(fread(Data, 298, 1, File) == 0) {
				fclose(File);
				File = NULL;
				return 0;
			}
			ID = SwapApID(Data);
			if((ID >= 0x200) && (ID < 0x37F)) UTC = GetLong4321(32);
		}
		return 1;
	}

	if(!AR) {
		GetCtrlVal(MainPanel, DECOM_UTCStart, Str);
		UTC1 = GetUTC(Str);
		GetCtrlVal(MainPanel, DECOM_UTCEnd, Str);
		UTC2 = GetUTC(Str);
		UTC=0;
		while(UTC < UTC1) {
			if(fread(Data, 298, 1, File) == 0) {
				fclose(File);
				File = NULL;
				return 0;
			}
			ID = SwapApID(Data);
			if((ID >= 0x200) && (ID < 0x37F)) UTC = GetLong4321(32);
		}
		if(((ID >= 0x200) && (ID < 0x300)) && (UTC > UTC2)) {
			fclose(File);
			File = NULL;
			return 0;
		}
		return 1;
	}
	
	if(fread(Data, 298, 1, File) == 0) {
		GetTime1(UTC, Str);
		ResetTextBox(MainPanel, DECOM_UTCEnd, Str);
		fclose(File);
		File = NULL;
		return 0;
	}
	ID = SwapApID(Data);
	if((ID >= 0x200) && (ID < 0x37F)) UTC = GetLong4321(32);
	return 1;
}


//---------------------------------------------------------------
// ApID Display

int GetApID()
{
	return Data[26] + ((Data[27]&7)<<8);
}	

int CVICALLBACK ScanApIDs (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	unsigned int ApIDCounter[256], Illegal, Legal, Displayed;
	unsigned int BadChecksum, Errors[256], SequenceErrors, ApIDSequence[256];
	unsigned int PLASTICApID, SpacecraftApID, BadPlastic;
	unsigned int ApIDCounterHET, ApIDCounterSEPT, ApIDCounterSIT, ApIDCounterLET;
	unsigned int ErrorsHET, ErrorsSEPT, ErrorsSIT, ErrorsLET;
	int i, ID, Sequence;
	
	switch (event)
		{
		case EVENT_COMMIT:
			SetWaitCursor(1);
			DisplayPanel ( ApIDPanel );
			for(i=0; i<256; i++) ApIDCounter[i] = Errors[i] = 0;
			Legal = Illegal = BadChecksum = SequenceErrors = 0;
			PLASTICApID = SpacecraftApID = BadPlastic = 0;
			while(GetData()) {
				ID = SwapApID(Data);
				Sequence = SwapSequence(Data);
				if(!LegalApID(ID)) Illegal++;
				else if(ID < 0x100) SpacecraftApID++;
				else {
					if(ID > 0x2FF) PLASTICApID++;
					else {
						if(ID > 0x27F) ID -= 0x80;
						ID -= 0x200;
						if((ApIDCounter[ID] != 0) &&
							(((ApIDSequence[ID]+1)&0x3FFF) != Sequence)) {
								SequenceErrors++;
								Errors[ID]++;
						}
						ApIDSequence[ID] = Sequence;
						ApIDCounter[ID]++;
						Legal++;
						if((Legal & 0xFF) == 0)
							SetCtrlVal(ApIDPanel, ApID_Total, Legal+Illegal);
					}
					if(ChecksumPacket(Data)) {
						BadChecksum++;
						Errors[ID]++;
					}
				}
			}
			
			for(ID=0x44, ApIDCounterLET=ErrorsLET=0; ID<=0x4A; ID++) {
				ApIDCounterLET += ApIDCounter[ID];
				ErrorsLET += Errors[ID];
			}
			for(ID=0x4E, ApIDCounterHET=ErrorsHET=0; ID<=0x55; ID++) {
				ApIDCounterHET += ApIDCounter[ID];
				ErrorsHET += Errors[ID];
			}
			for(ID=0x58, ApIDCounterSEPT=ErrorsSEPT=0; ID<=0x59; ID++) {
				ApIDCounterSEPT += ApIDCounter[ID];
				ErrorsSEPT += Errors[ID];
			}
			for(ID=0x5D, ApIDCounterSIT=ErrorsSIT=0; ID<=0x68; ID++) {
				ApIDCounterSIT += ApIDCounter[ID];
				ErrorsSIT += Errors[ID];
			}

			SetCtrlVal(ApIDPanel, ApID_200, ApIDCounter[0]);
			SetCtrlVal(ApIDPanel, ApID_201, ApIDCounter[1]);
			SetCtrlVal(ApIDPanel, ApID_202, ApIDCounter[2]);
			SetCtrlVal(ApIDPanel, ApID_203, ApIDCounter[3]);
			SetCtrlVal(ApIDPanel, ApID_208, ApIDCounter[8]);
			SetCtrlVal(ApIDPanel, ApID_209, ApIDCounter[9]);
			SetCtrlVal(ApIDPanel, ApID_210, ApIDCounter[0x10]);
			SetCtrlVal(ApIDPanel, ApID_211, ApIDCounter[0x11]);
			SetCtrlVal(ApIDPanel, ApID_212, ApIDCounter[0x12]);
			SetCtrlVal(ApIDPanel, ApID_213, ApIDCounter[0x13]);
			SetCtrlVal(ApIDPanel, ApID_214, ApIDCounter[0x14]);
			SetCtrlVal(ApIDPanel, ApID_215, ApIDCounter[0x15]);
			SetCtrlVal(ApIDPanel, ApID_220, ApIDCounter[0x20]);
			SetCtrlVal(ApIDPanel, ApID_221, ApIDCounter[0x21]);
			SetCtrlVal(ApIDPanel, ApID_222, ApIDCounter[0x22]);
			SetCtrlVal(ApIDPanel, ApID_223, ApIDCounter[0x23]);
			SetCtrlVal(ApIDPanel, ApID_240, ApIDCounter[0x40]);
			SetCtrlVal(ApIDPanel, ApID_241, ApIDCounter[0x41]);
			SetCtrlVal(ApIDPanel, ApID_LET, ApIDCounterLET);
			SetCtrlVal(ApIDPanel, ApID_HET, ApIDCounterHET);
			SetCtrlVal(ApIDPanel, ApID_SEPT, ApIDCounterSEPT);
			SetCtrlVal(ApIDPanel, ApID_SIT, ApIDCounterSIT);
			SetCtrlVal(ApIDPanel, ApID_270, ApIDCounter[0x70]);
			SetCtrlVal(ApIDPanel, ApID_PLASTIC, PLASTICApID);
			SetCtrlVal(ApIDPanel, ApID_Spacecraft, SpacecraftApID);
			
			SetCtrlVal(ApIDPanel, ApID_200E, Errors[0]);
			SetCtrlVal(ApIDPanel, ApID_201E, Errors[1]);
			SetCtrlVal(ApIDPanel, ApID_202E, Errors[2]);
			SetCtrlVal(ApIDPanel, ApID_203E, Errors[3]);
			SetCtrlVal(ApIDPanel, ApID_208E, Errors[8]);
			SetCtrlVal(ApIDPanel, ApID_209E, Errors[9]);
			SetCtrlVal(ApIDPanel, ApID_210E, Errors[0x10]);
			SetCtrlVal(ApIDPanel, ApID_211E, Errors[0x11]);
			SetCtrlVal(ApIDPanel, ApID_212E, Errors[0x12]);
			SetCtrlVal(ApIDPanel, ApID_213E, Errors[0x13]);
			SetCtrlVal(ApIDPanel, ApID_214E, Errors[0x14]);
			SetCtrlVal(ApIDPanel, ApID_215E, Errors[0x15]);
			SetCtrlVal(ApIDPanel, ApID_220E, Errors[0x20]);
			SetCtrlVal(ApIDPanel, ApID_221E, Errors[0x21]);
			SetCtrlVal(ApIDPanel, ApID_222E, Errors[0x22]);
			SetCtrlVal(ApIDPanel, ApID_223E, Errors[0x23]);
			SetCtrlVal(ApIDPanel, ApID_240E, Errors[0x40]);
			SetCtrlVal(ApIDPanel, ApID_241E, Errors[0x41]);
			SetCtrlVal(ApIDPanel, ApID_LETE, ErrorsLET);
			SetCtrlVal(ApIDPanel, ApID_HETE, ErrorsHET);
			SetCtrlVal(ApIDPanel, ApID_SEPTE, ErrorsSEPT);
			SetCtrlVal(ApIDPanel, ApID_SITE, ErrorsSIT);
			SetCtrlVal(ApIDPanel, ApID_270E, Errors[0x70]);
			
			Displayed = ApIDCounter[0]+ApIDCounter[1]+ApIDCounter[2]+ApIDCounter[3]+ApIDCounter[8]+ApIDCounter[9]+
			  ApIDCounter[0x10]+ApIDCounter[0x11]+ApIDCounter[0x12]+ApIDCounter[0x13]+ApIDCounter[0x14]+ApIDCounter[0x15]+
			  ApIDCounter[0x20]+ApIDCounter[0x21]+ApIDCounter[0x22]+ApIDCounter[0x23]+ApIDCounter[0x70]+
			  ApIDCounter[0x40]+ApIDCounter[0x41]+ApIDCounterLET+ApIDCounterHET+ApIDCounterSEPT+ApIDCounterSIT;
			SetCtrlVal(ApIDPanel, ApID_Illegal, Illegal);
			SetCtrlVal(ApIDPanel, ApID_Other, Legal - Displayed);
			SetCtrlVal(ApIDPanel, ApID_Total, Legal+Illegal);
			SetCtrlVal(ApIDPanel, ApID_BadChecksum, BadChecksum);
			SetCtrlVal(ApIDPanel, ApID_SequenceError, SequenceErrors);
			SetWaitCursor(0);
			break;
		}
	return 0;
}


//----------------------------------------------------------
// Packet Display

int PacketNumber;

static void DisplayHKPPacket(char *Data);
static void DisplayMagPacket(char *Data);
static void DisplaySWEAFHKP(char *Data);

void DisplayPacket(unsigned char *Data)
{
	char String[1000];
	int i, Sum;
	char *s;
	
	PacketNumber = ftell(DumpFile)/298 -1;
	SetCtrlVal(PacketPanel, PACKET_PacketNum, PacketNumber);
	sprintf(String, "%04X %02X %02X %02X %04X %04X %08X %04X %02X\n%02X %02X %02X %02X %02X %02X %02X %02X %02X %02X",
		GetWord21(0), GetByte(2), GetByte(3), GetByte(4),
		GetWord21(5), GetWord21(7), GetLong4321(9), GetWord21(13), GetByte(15),
		GetByte(16), GetByte(17), GetByte(18), GetByte(19), GetByte(20),
		GetByte(21), GetByte(22), GetByte(23), GetByte(24), GetByte(25));
	ResetTextBox(PacketPanel, PACKET_GRHeader, String);	
	sprintf(String, "%04X %04X %04X %08X.%02X",
		GetWord(26), GetWord(28), GetWord(30), GetLong3412(32), Data[37]);
	ResetTextBox(PacketPanel, PACKET_CCSDSHeader, String);
	SetCtrlVal(PacketPanel, PACKET_ApID, GetApID());
	ResetTextBox(PacketPanel, PACKET_Time, GetTime());
	
	for(i=0, s=String; i<272; i++) {
		sprintf(s, "%02X  ", GetByte(26+i));
		if(i%16 == 15) {
			*(s+2) = '\n';
			s += 3;
		}
		else if(i%4 == 3) s += 4;
		else s += 3;
	}
	ResetTextBox(PacketPanel, PACKET_Data, String);
	Sum = ChecksumPacket(Data);
	SetCtrlVal(PacketPanel, PACKET_Sum, Sum);
	SetCtrlAttribute (PacketPanel, PACKET_Sum, ATTR_TEXT_BGCOLOR, Sum?0x00FF0000:0x0000FF00);
	DisplayHKPPacket(Data);
	DisplayMagPacket(Data);
	DisplaySWEAFHKP(Data);
}


int CVICALLBACK DisplayPackets (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			DisplayPanel ( PacketPanel );
			break;
		}
	return 0;
}


int CVICALLBACK NextPacket (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int N;
	
	switch (event)
		{
		case EVENT_COMMIT:
			if(fread(Data, 298, 1, DumpFile)==0) {
				N = (ftell(DumpFile)-297)/298;
				fseek(DumpFile, N*298, SEEK_SET); 
				fread(Data, 298, 1, DumpFile);
			}
			SwapTelemetryBytes(Data);
			DisplayPacket(Data);
			break;
		}
	return 0;
}

int CVICALLBACK BackPacket (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			if(PacketNumber) {
				fseek(DumpFile, -298*2, SEEK_CUR);
				fread(Data, 298, 1, DumpFile);
				SwapTelemetryBytes(Data);
			}
			DisplayPacket(Data);
			break;
		}
	return 0;
}

int CVICALLBACK PacketNum (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(panel, control, &PacketNumber);
			fseek(DumpFile, 298*PacketNumber, SEEK_SET);
			fread(Data, 298, 1, DumpFile);
			SwapTelemetryBytes(Data);
			DisplayPacket(Data);
			break;
		}
	return 0;
}

#define FindBadChecksum	0
#define FindIllegalApID 1
#define FindSequenceError 2
#define FindMAGIFC 3

int CVICALLBACK FindNext (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int Find, N;
	unsigned int ApIDSequence[256];
	int i, ID, Sequence, InSequence, IFC, IFCLast;
	
	switch (event)
		{
		case EVENT_COMMIT:
			SetWaitCursor(1);
			GetCtrlVal(panel, PACKET_FindSelect, &Find);
			if(Find == FindBadChecksum) {		// Find next bad checksum
				while((N=fread(Data, 298, 1, DumpFile)) && ((SwapApID(Data) < 0x100) || (ChecksumPacket(Data)==0)));
			}
			else if(Find == FindIllegalApID) {
				while((N=fread(Data, 298, 1, DumpFile)) && LegalApID(SwapApID(Data)));
			}
			else if(Find == FindSequenceError) {
				for(i=0; i<256; i++) ApIDSequence[i] = 0;
				InSequence = 1;
				while(InSequence && (N=fread(Data, 298, 1, DumpFile))) {
					Sequence = SwapSequence(Data);
					ID = SwapApID(Data);
					if(LegalApID(ID) && (ID > 0x100) && (ID < 0x300)) {
						if(ID > 0x27F) ID -= 0x80;
						ID -= 0x200;
						if(ApIDSequence[ID] != 0)
							if(((ApIDSequence[ID]+1)&0x3FFF) != Sequence)
								InSequence = 0;
						ApIDSequence[ID] = Sequence;
					}
				}
			}
			else if(Find == FindMAGIFC) {
				IFC = IFCLast = -1;
				while((IFC == IFCLast) && (N=fread(Data, 298, 1, DumpFile))) {
					IFCLast = IFC;
					if(SwapApID(Data) == 0x208) IFC = GetSwappedByte(26+16)&0x40;
					if(IFCLast == -1) IFCLast = IFC;
				}
			}

			else {
				while((N=fread(Data, 298, 1, DumpFile)) && (Find != SwapApID(Data)));
			}
			if(N == 0) {
				N = (ftell(DumpFile)-297)/298;
				fseek(DumpFile, N*298, SEEK_SET); 
				fread(Data, 298, 1, DumpFile);
			}
			SwapTelemetryBytes(Data);
			DisplayPacket(Data);
			SetWaitCursor(0);
			break;
		}
	return 0;
}

//------------------------------------------------
// Housekeeping

int CVICALLBACK DisplayHKP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			DisplayPanel ( HKPPanel );
			break;
		}
	return 0;
}

int CVICALLBACK BackHKP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int Was, Found;
	
	switch (event)
		{
		case EVENT_COMMIT:
			Found = 0;
			while(Found == 0 && (ftell(DumpFile) > 298)) {
				fseek(DumpFile, -298*2, SEEK_CUR);
				fread(Data, 298, 1, DumpFile);
				Found = (SwapApID(Data) == 0x200);
			}
			SwapTelemetryBytes(Data);
			DisplayPacket(Data);
			break;
		}
	return 0;
}

#define FindError 0
#define FindCommand 1
#define FindErrorNot12 2

int CVICALLBACK FindNextHKP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	unsigned int N, Find, Current, NoNewError;
	
	switch (event)
		{
		case EVENT_COMMIT:
			SetWaitCursor(1);
			GetCtrlVal(panel, HKP_FindSelect, &Find);
			if(Find == FindError) {		// Find next bad checksum
				Current = GetByte(26+20);
				while((N=fread(Data, 298, 1, DumpFile)) &&
					(SwapApID(Data) == 0x200) &&
					(Current == GetSwappedByte(26+20)));
			}
			else if(Find == FindCommand) {
				Current = GetByte(26+21);
				while((N=fread(Data, 298, 1, DumpFile)) &&
					(SwapApID(Data) == 0x200) &&
					(Current == GetSwappedByte(26+21)));
			}
			else if(Find = FindErrorNot12) {
				Current = GetByte(26+20);
				NoNewError=1;
				while(NoNewError && (N=fread(Data, 298, 1, DumpFile))) {
					if((SwapApID(Data) == 0x200) &&
						(Current != GetSwappedByte(26+20)) &&
						(GetSwappedByte(26+18) != 12)) 
							NoNewError=0;
				}
			}
			if(N == 0) {
				N = (ftell(DumpFile)-297)/298;
				fseek(DumpFile, N*298, SEEK_SET); 
				fread(Data, 298, 1, DumpFile);
			}
			SwapTelemetryBytes(Data);
			DisplayPacket(Data);
			SetWaitCursor(0);
			break;
		}
	return 0;
}

int CVICALLBACK NextHKP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int N;
	
	switch (event)
		{
		case EVENT_COMMIT:
			while((N=fread(Data, 298, 1, DumpFile)) && (SwapApID(Data) != 0x200));
			if(N == 0) {
				N = (ftell(DumpFile)-297)/298;
				fseek(DumpFile, N*298, SEEK_SET); 
				fread(Data, 298, 1, DumpFile);
			}
			SwapTelemetryBytes(Data);
			DisplayPacket(Data);

			break;
		}
	return 0;
}

int Decompress(int x)
{
	int e,m;
	e = (x>>4)&0x0f;
	m = x&0x0f;
	if(e<2) return m;
	else return (m+16)<<(e-1);
}

static void DisplayHKPPacket(char *Data)
{
	int Sum, T;
	char String[100];
	
	if(GetApID() != 0x200) return;
	SetCtrlVal(HKPPanel, HKP_PacketNum, PacketNumber);
	Sum = ChecksumPacket(Data);
	SetCtrlVal(HKPPanel, HKP_Sum, Sum);
	SetCtrlAttribute (HKPPanel, HKP_Sum, ATTR_TEXT_BGCOLOR, Sum?0x00FF0000:0x0000FF00);
	ResetTextBox(HKPPanel, HKP_Time, GetTime());
	SetCtrlVal(HKPPanel, HKP_PollTimeMax, GetWord(26+16)*32./1000000.);
	SetCtrlVal(HKPPanel, HKP_PollTimeAvg, GetWord(26+18)*32./1000000.);
	SetCtrlVal(HKPPanel, HKP_TimeJitter, Decompress(GetByte(26+20))/65536.);

	T = GetWord(26+12);
	sprintf(String, "%2d:%02d:%02d", T>>12, (T>>6)&0x3f, T&0x3F);
	ResetTextBox(HKPPanel, HKP_SampleTime, String);
	SetCtrlVal(HKPPanel, HKP_ErrorCount, GetByte(26+23));
	SetCtrlVal(HKPPanel, HKP_ErrorCode, GetByte(26+21));
	SetCtrlVal(HKPPanel, HKP_ErrorData, GetByte(26+22));
	SetCtrlVal(HKPPanel, HKP_CommandCount, GetWord(26+24));
	SetCtrlVal(HKPPanel, HKP_CommandApID, GetByte(26+26));
	SetCtrlVal(HKPPanel, HKP_CommandSeq, GetByte(26+27));

}

// Housekeeping scan

int CVICALLBACK ScanHKP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	unsigned int PacketCount, HKPCount;
	unsigned int IDError, SequenceError, SizeError, ChecksumError, SWVError, HWVError;
	int i, ID, Sequence,Size;
	int SequenceLast, UTCs, UTCsLast, UTCf, UTCfLast, T, TLast, SWVersion, HWVersion;
	double DUTC, DUTCMax, DUTCMin, DUTCAvg;
	int DT, DTMax, DTMin, DTAvg;
	int IPollTimeMax, IPollTimeMaxMax, ITimeJitter, ITimeJitterMax, ITimeJitterSat;
	int ICommandCount, IErrorCount, ICommandCountLast, IErrorCountLast, ICommandTotal, IErrorTotal;
	int PCommandCount, PCommandCountLast, PCommandTotal;
	double IPollTimeAvg;
	char String[50], *TimeStr;
	
	switch (event)
		{
		case EVENT_COMMIT:
			SetWaitCursor(1);
			DisplayPanel ( ScanHKPPanel );
			PacketCount=HKPCount=0;
			ChecksumError=IDError=SequenceError=SizeError=0;
			SWVError=HWVError=0;
			DUTCMax = DUTCAvg = 0.;
			DUTCMin = 1000000.;
			DTMax = DTAvg = 0;
			DTMin = 100000;
			SequenceLast=0;
			ICommandTotal = PCommandTotal = IErrorTotal = 0;
			IPollTimeMaxMax = IPollTimeAvg = ITimeJitterMax = ITimeJitterSat = 0;
			while(GetData()) {
				PacketCount++;
				if(((ID = SwapWord(Data, 0))&0x7FF) == 0x200) {
					HKPCount++;
					if(ID != 0x0A00) IDError++;
					Sequence = SwapWord(Data,1);
					if(SequenceLast != 0) {
						if((Sequence&0xC000) != 0xC000) SequenceError++;
						else if(((SequenceLast+1)&0x3FFF) != (Sequence&0x3FFF)) SequenceError++;
					}
					Size = SwapWord(Data, 2);
					if(Size != 0x109) SizeError++;
					if(ChecksumPacket(Data)) ChecksumError++;
					SwapTelemetryBytes(Data);
					UTCs = GetLong3412(32);
					UTCf = (GetByte(37)<<8) + GetByte(36);
					T = GetWord(26+12);
					ICommandCount = GetWord(24+26);
					PCommandCount = GetWord(28+26);
					IErrorCount = GetByte(23+26);
					if(SequenceLast == 0) {
						ResetTextBox(ScanHKPPanel, SCANHKP_UTCStart, GetTime());
						SetCtrlVal(ScanHKPPanel, SCANHKP_SWVersion, SWVersion=GetByte(26+14));
						SetCtrlVal(ScanHKPPanel, SCANHKP_HWVersion, HWVersion=GetByte(26+15));
						sprintf(String, "%2d:%02d:%02d", T>>12, (T>>6)&0x3f, T&0x3F);
						ResetTextBox(ScanHKPPanel, SCANHKP_SampleTimeStart, String);
					}
					else {
						DUTC = (UTCs-UTCsLast) + (UTCf-UTCfLast)/65536.;
						if(DUTC > DUTCMax) DUTCMax = DUTC;
						if(DUTC < DUTCMin) DUTCMin = DUTC;
						DUTCAvg += DUTC;
						if(SWVersion != GetByte(26+14)) SWVError++;
						if(HWVersion != GetByte(26+15)) HWVError++;
						DT = ((T&0x3F)-(TLast&0x3F))+(((T>>6)&0x3F)-((TLast>>6)&0x3F))*60+
							((T>>12)-(TLast>>12))*3600;
						if(((T>>12)==0) && ((TLast>>12) == 15)) DT += 16*3600;
						if(DT > DTMax) DTMax = DT;
						if(DT < DTMin) DTMin = DT;
						DTAvg += DT;
						IErrorTotal += IErrorCount - IErrorCountLast;
						ICommandTotal += ICommandCount - ICommandCountLast;
						PCommandTotal += PCommandCount - PCommandCountLast;
					}
					IPollTimeMax = GetWord(26+16);
					if(IPollTimeMax > IPollTimeMaxMax) IPollTimeMaxMax = IPollTimeMax;
					IPollTimeAvg += GetWord(26+18)*.000032;
					ITimeJitter = GetByte(26+20);
					if(ITimeJitter == 0xFF) ITimeJitterSat++;
					else if(ITimeJitter > ITimeJitterMax) ITimeJitterMax = ITimeJitter;
					UTCsLast = UTCs;
					UTCfLast = UTCf;
					TLast = T;
					TimeStr = GetTime();
					SequenceLast = Sequence;
					ICommandCountLast = ICommandCount;
					IErrorCountLast = IErrorCount;
					PCommandCountLast = PCommandCount;
				}
			}
			ResetTextBox(ScanHKPPanel, SCANHKP_UTCEnd, TimeStr);
			sprintf(String, "%2d:%02d:%02d", T>>12, (T>>6)&0x3f, T&0x3F);
			ResetTextBox(ScanHKPPanel, SCANHKP_SampleTimeEnd, String);
			SetCtrlVal(ScanHKPPanel, SCANHKP_Packets, PacketCount);
			SetCtrlVal(ScanHKPPanel, SCANHKP_HKP, HKPCount);
			SetCtrlVal(ScanHKPPanel, SCANHKP_IDError, IDError);
			SetCtrlVal(ScanHKPPanel, SCANHKP_SequenceError, SequenceError);
			SetCtrlVal(ScanHKPPanel, SCANHKP_SizeError, SizeError);
			SetCtrlVal(ScanHKPPanel, SCANHKP_ChecksumError, ChecksumError);
			SetCtrlVal(ScanHKPPanel, SCANHKP_DUTCMax, DUTCMax);
			SetCtrlVal(ScanHKPPanel, SCANHKP_DUTCMin, DUTCMin);
			SetCtrlVal(ScanHKPPanel, SCANHKP_DUTCAvg, DUTCAvg/(HKPCount-1));
			SetCtrlVal(ScanHKPPanel, SCANHKP_SWVError, SWVError);
			SetCtrlVal(ScanHKPPanel, SCANHKP_DTMax, DTMax);
			SetCtrlVal(ScanHKPPanel, SCANHKP_DTMin, DTMin);
			SetCtrlVal(ScanHKPPanel, SCANHKP_DTAvg, (double)DTAvg/(HKPCount-1));
			SetCtrlVal(ScanHKPPanel, SCANHKP_HWVError, HWVError);
			SetCtrlVal(ScanHKPPanel, SCANHKP_IPollTimeMax, IPollTimeMaxMax*.000032);
			SetCtrlVal(ScanHKPPanel, SCANHKP_IPollTimeAvg, IPollTimeAvg/HKPCount);
			SetCtrlVal(ScanHKPPanel, SCANHKP_ITimeJitter, Decompress(ITimeJitterMax)/65536.);
			SetCtrlVal(ScanHKPPanel, SCANHKP_ITimeJitterSat, ITimeJitterSat);
			SetCtrlVal(ScanHKPPanel, SCANHKP_ICommandCount, ICommandTotal);
			SetCtrlVal(ScanHKPPanel, SCANHKP_PCommandCount, PCommandTotal);
			SetCtrlVal(ScanHKPPanel, SCANHKP_IErrorCount, IErrorTotal);
			
			SetWaitCursor(0);
			break;
		}
	return 0;
}


//---------------------------------------------------------------
// Timing display
//


int CVICALLBACK DisplayTiming (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			DisplayPanel ( TimingPanel );
			break;
		}
	return 0;
}


int dThist[1000000];

int CVICALLBACK StartTiming (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int FindApID, ID, Packets, T, Tf, Tff, dT, dTmax, dTmin, TL, TfL, TffL, i;
	int Time16;
	char Str[20];
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(panel, TIMING_APID, &FindApID);
			GetCtrlVal(panel, TIMING_TIME16, &Time16);
			DeleteGraphPlot (panel, TIMING_GRAPH, -1, VAL_IMMEDIATE_DRAW);
			SetWaitCursor(1);
			Packets = 0;
			for(i=0; i<1000000; i++) dThist[i] = 0;
			dTmax = 0;
			dTmin = 999999;
			while(GetData()) if(SwapApID(Data) == FindApID) {
				SwapTelemetryBytes(Data);
				T = GetLong3412(32);
				Tf = GetByte(37);
				Tff = GetByte(36);
				if(Packets) {
					dT = (T-TL)*256 + (Tf-TfL);
					if(Time16) dT = dT*256 + Tff-TffL;
					if(dT<0) dT=0;
					else if (dT >= 1000000) dT = 999999;
					dThist[dT]++;
					if(dT > dTmax) dTmax = dT;
					if (dT < dTmin) dTmin = dT;
				}
				TL = T;
				TfL = Tf;
				TffL = Tff;
				Packets++;
			}
			sprintf(Str, "%d Packets", Packets);
			SetCtrlVal(panel, TIMING_PACKETS, Str);
			if((dTmin -= 10) < 0) dTmin=0;
			if((dTmax += 10) > 999999) dTmax = 999999;
			if(Packets) {
				SetCtrlAttribute(panel, TIMING_GRAPH, ATTR_ENABLE_ZOOM_AND_PAN, 0);
				SetCtrlAttribute(panel, TIMING_GRAPH, ATTR_XAXIS_OFFSET, 
					Time16?(dTmin+0.5)/65536.:(dTmin+0.5)/256.);
				SetCtrlAttribute (panel, TIMING_GRAPH, ATTR_XAXIS_GAIN,
					Time16?1./65536.:1./256.);
				PlotY (panel, TIMING_GRAPH, &dThist[dTmin], dTmax-dTmin, VAL_UNSIGNED_INTEGER, VAL_THIN_STEP,
       				VAL_SIMPLE_DOT, VAL_SOLID, 1, VAL_RED);
       			SetCtrlAttribute(panel, TIMING_GRAPH, ATTR_ENABLE_ZOOM_AND_PAN, 1);
			}
			SetWaitCursor(0);
			break;
		}
	return 0;
}

//------------------------------------------------
// MAG


int CVICALLBACK PlotMagPacket (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			DisplayPanel ( MagPanel );
			break;
		}
	return 0;
}


#define SIZE 1000000
static float MagX[SIZE], MagY[SIZE], MagZ[SIZE], AvgX, AvgY, AvgZ;
static char *MagTime;

static void DisplayMagPacket(char *Data)
{
	int i, ACDC;
	float Gain = 1.;	//5./32768.;
	
	if((GetApID()&0xFFE) != 0x208) return;
	SetCtrlVal(MagPanel, MAG_PacketNum, PacketNumber);
	DeleteGraphPlot (MagPanel, MAG_GRAPH, -1, VAL_IMMEDIATE_DRAW);
	GetCtrlVal(MagPanel, MAG_ACDC, &ACDC);
	MagTime = GetTimeL();
	
	for(i=0; i<42; i++) MagX[i] = Gain*((int)GetWord(26+18+6*i)-32768);
	if(ACDC) {
		for(i=0, AvgX=0.; i<42; i++) AvgX += MagX[i];
		for(i=0; i<42; i++) MagX[i] -= AvgX/42.;
	}
	PlotY (MagPanel, MAG_GRAPH, MagX, 42, VAL_FLOAT, VAL_CONNECTED_POINTS,
       				VAL_SMALL_CROSS, VAL_SOLID, 1, VAL_RED);
       				
	for(i=0; i<42; i++) MagY[i] = Gain*((int)GetWord(26+18+6*i+2)-32768);
	if(ACDC) {
		for(i=0, AvgY=0.; i<42; i++) AvgY += MagY[i];
		for(i=0; i<42; i++) MagY[i] -= AvgY/42.;
	}
	PlotY (MagPanel, MAG_GRAPH, MagY, 42, VAL_FLOAT, VAL_CONNECTED_POINTS,
       				VAL_SMALL_CROSS, VAL_SOLID, 1, VAL_GREEN);
       				
	for(i=0; i<42; i++) MagZ[i] = Gain*((int)GetWord(26+18+6*i+4)-32768);
	if(ACDC) {
		for(i=0, AvgZ=0.; i<42; i++) AvgZ += MagZ[i];
		for(i=0; i<42; i++) MagZ[i] -= AvgZ/42.;
	}
	PlotY (MagPanel, MAG_GRAPH, MagZ, 42, VAL_FLOAT, VAL_CONNECTED_POINTS,
       				VAL_SMALL_CROSS, VAL_SOLID, 1, VAL_BLUE);
	
}

int CVICALLBACK SaveMag (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	FILE *File;
	char OutName[100];
	int ACDC, j;
	
	switch (event)
		{
		case EVENT_COMMIT:
			strcpy(OutName, FileName);
			strcpy(strrchr(OutName, '.'), ".txt");
			File = fopen(OutName, "a");
			GetCtrlVal(MagPanel, MAG_ACDC, &ACDC);
			fprintf(File, "STEREO IMPACT MAG Data\n");
			fprintf(File, " Data extracted from telemetry file %s, Packet %d, %s, %s\n\n",
				FileName, PacketNumber, MagTime, ACDC?"AC":"DC");
			fprintf(File, "Sample\tX\tY\tZ\n");
			for(j=0; j<42; j++) {
				fprintf(File, "%2d\t%8.1f\t%8.1f\t%8.1f\n", j, MagX[j], MagY[j], MagZ[j]); 
			}
			fclose(File);
			break;
		}
	return 0;
}

static double UTCf[SIZE];

int CVICALLBACK PlotMAG (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i, j, Dat, NS, T0, Tf, Rate, AS, ApIDSelect;
	float Val;
	double Min, Max;
	char Str[50];

	switch (event) {
		case EVENT_COMMIT:
			DisplayPanel ( PlotMAGPanel );
			DeleteGraphPlot (PlotMAGPanel, PLOTMAG_GRAPH, -1,
                 VAL_IMMEDIATE_DRAW);
            ClearAxisItems (PlotMAGPanel, PLOTMAG_GRAPH, VAL_XAXIS );
			SetWaitCursor(1);
			GetCtrlVal(PlotMAGPanel, PLOTMAG_BurstSelect, &ApIDSelect);
			if(ApIDSelect) ApIDSelect = 0x209;
			else ApIDSelect = 0x208;
			NS = 0;
			while(GetData() && (NS<SIZE)) if((SwapWord(Data, 0)&0x7FF) == ApIDSelect) {
				SwapTelemetryBytes(Data);
				T0 = GetLong3412(32);
				Tf = GetWord(36);
				Rate = GetByte(26+13);
				for(i=0; i<42; i++) {
					UTCf[NS] = T0 + Tf/65536. + (i*Rate/32.);
					MagX[NS] = (float)((int)GetWord(26+18+6*i)-32768);
					MagY[NS] = (float)((int)GetWord(26+18+6*i+2)-32768);
					MagZ[NS] = (float)((int)GetWord(26+18+6*i+4)-32768);
					NS++;
				}
			}

			GetCtrlVal(PlotMAGPanel, PLOTMAG_AutoScale, &AS);
			if(AS) {
				Min = 1.E6;
				Max = -1.E6;
				for(i=0; i<NS; i++) {
					if (MagX[i] < Min) Min = MagX[i];
					if (MagX[i] > Max) Max = MagX[i];
					if (MagY[i] < Min) Min = MagY[i];
					if (MagY[i] > Max) Max = MagY[i];
					if (MagZ[i] < Min) Min = MagZ[i];
					if (MagZ[i] > Max) Max = MagZ[i];
				}
				SetCtrlVal(PlotMAGPanel, PLOTMAG_MIN, Min);
				SetCtrlVal(PlotMAGPanel, PLOTMAG_MAX, Max);
			}
			else {
				GetCtrlVal(PlotMAGPanel, PLOTMAG_MIN, &Min);
				GetCtrlVal(PlotMAGPanel, PLOTMAG_MAX, &Max);
			}
			
			SetAxisScalingMode (PlotMAGPanel, PLOTMAG_GRAPH,
                    VAL_LEFT_YAXIS, VAL_MANUAL, Min, Max);
			SetAxisScalingMode (PlotMAGPanel, PLOTMAG_GRAPH,
                    VAL_XAXIS, VAL_MANUAL, UTCf[0], UTCf[NS-1]);
                    
    		j=0; 
            for(i=1; i <= NS; i++) {
            	if(((UTCf[i]-UTCf[i-1]) > 20.) || (i == NS)) {
     	
					PlotXY (PlotMAGPanel, PLOTMAG_GRAPH, &UTCf[j], &MagX[j], i-j,
        				VAL_DOUBLE, VAL_FLOAT, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, VAL_RED);
					PlotXY (PlotMAGPanel, PLOTMAG_GRAPH, &UTCf[j], &MagY[j], i-j,
        				VAL_DOUBLE, VAL_FLOAT, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, VAL_GREEN);
					PlotXY (PlotMAGPanel, PLOTMAG_GRAPH, &UTCf[j], &MagZ[j], i-j,
        				VAL_DOUBLE, VAL_FLOAT, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, VAL_BLUE);
        				
        			j=i;
        		}
        	}

			InsertAxisItem (PlotMAGPanel, PLOTMAG_GRAPH, VAL_XAXIS, -1,
				GetTime1((int)UTCf[0], Str), UTCf[0]);
			InsertAxisItem (PlotMAGPanel, PLOTMAG_GRAPH, VAL_XAXIS, -1,
				GetTime1((int)UTCf[NS-1], Str), UTCf[NS-1]);
			
			SetWaitCursor(0);
			break;
		}

	return 0;
}

// SWEA FHKP

int CVICALLBACK PlotSWEAFHKP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			DisplayPanel ( FHKPPanel );
			break;
		}
	return 0;
}

static int Y[337];
static int Zoom=1, Ymin, Ymax, Offset=0;

static void DisplaySWEAFHKP(char *Data)
{
	int i, Sample, Source;
	
	if((GetApID()&0xFFF) != 0x214) return;
	Sample = 336-GetWord(26+12);
	if(Sample < 0 || Sample > 336) return;

	SetCtrlVal(FHKPPanel, SWEAFHKP_PacketNum, PacketNumber);
	SetCtrlVal(FHKPPanel, SWEAFHKP_Source, Source=GetWord(26+14)>>12);
	DeleteGraphPlot (FHKPPanel, SWEAFHKP_GRAPH, -1, VAL_IMMEDIATE_DRAW);

	for(i=0; i<128; i++) if(i+Sample < 336) {
		Y[i+Sample] = GetWord(26+14+2*i)&0xFFF;
		if(Source < 8) Y[i+Sample] = 2048-Y[i+Sample];
	}
	Ymin = 5000;
	Ymax = -5000;
	for(i=0; i<336; i++) {
		if(Y[i] < Ymin) Ymin = Y[i];
		if(Y[i] > Ymax) Ymax = Y[i];
	}
	SetAxisScalingMode (FHKPPanel, SWEAFHKP_GRAPH, VAL_LEFT_YAXIS,
                    VAL_MANUAL, Ymin/Zoom, Ymax/Zoom);
	PlotY (FHKPPanel, SWEAFHKP_GRAPH, Y, 336, VAL_INTEGER, VAL_CONNECTED_POINTS,
       				VAL_SMALL_CROSS, VAL_SOLID, 1, VAL_BLACK);

}

int CVICALLBACK ZoomOut (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{

	switch (event)
		{
		case EVENT_COMMIT:
			if(Zoom != 1) Zoom/=2;
			SetAxisScalingMode (FHKPPanel, SWEAFHKP_GRAPH, VAL_LEFT_YAXIS,
                    VAL_MANUAL, Ymin+Offset, Ymin+Offset+(Ymax-Ymin)/Zoom);
			break;
		}
	return 0;
}

int CVICALLBACK ZoomIn (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			Zoom *= 2;
			SetAxisScalingMode (FHKPPanel, SWEAFHKP_GRAPH, VAL_LEFT_YAXIS,
                    VAL_MANUAL, Ymin+Offset, Ymin+Offset+(Ymax-Ymin)/Zoom);
			break;
		}
	return 0;
}

int CVICALLBACK MoveDown (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			Offset -= (Ymax-Ymin)/(Zoom*2);
			SetAxisScalingMode (FHKPPanel, SWEAFHKP_GRAPH, VAL_LEFT_YAXIS,
                    VAL_MANUAL, Ymin+Offset, Ymin+Offset+(Ymax-Ymin)/Zoom);
			break;
		}
	return 0;
}

int CVICALLBACK MoveUp (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			Offset += (Ymax-Ymin)/(Zoom*2);
			SetAxisScalingMode (FHKPPanel, SWEAFHKP_GRAPH, VAL_LEFT_YAXIS,
                    VAL_MANUAL, Ymin+Offset, Ymin+Offset+(Ymax-Ymin)/Zoom);
			break;
		}
	return 0;
}

int CVICALLBACK Auto (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			Offset = 0;
			Zoom = 1;
			SetAxisScalingMode (FHKPPanel, SWEAFHKP_GRAPH, VAL_LEFT_YAXIS,
                    VAL_MANUAL, Ymin+Offset, Ymin+Offset+(Ymax-Ymin)/Zoom);
			break;
		}
	return 0;
}


// Burst Timing scan

void ScanBurst1()
{
	unsigned int PacketCount[5], SequenceLast[5], UTCsLast[5], PktNoLast[5];
	unsigned int UTCf, UTCs, T, ID, Sequence, Total, i, SBSel;
	static unsigned int BurstID[4] = { 0x209, 0x211, 0x221, 0x223 };
	char Str[100], T1[50], T2[50];
	
	SetWaitCursor(1);
	DisplayPanel ( ScanBurstPanel );
	GetCtrlVal(ScanBurstPanel, SCANBURST_Select, &SBSel);
	ResetTextBox(ScanBurstPanel, SCANBURST_Text, "");
	for(i=0; i<5; i++) SequenceLast[i] = PacketCount[i] = 0;
	Total = 0;
	
	while(GetData()) {
		Total++;
		ID = SwapApID(Data);
		for(i=0; (i<4) && (ID != BurstID[i]); i++);
		PacketCount[i]++;
		if(i<4) {
			if((ID == SBSel) || (SBSel == 0)) {
				SwapTelemetryBytes(Data);
				UTCs = GetLong3412(32);
				UTCf = (GetByte(37)<<8) + GetByte(36);
				T = GetWord(26+12);
				Sequence = GetWord(26+2);
				if((SequenceLast[i] != 0) && ((SequenceLast[i]+1)&0x3FFF) != (Sequence&0x3FFF)) {
					sprintf(Str, "%6d Skip ApID %03Xh %s - %s #%d\n", PktNoLast[i], ID, GetTime1(UTCsLast[i],T1),
						GetTime1(UTCs,T2), (Sequence-(SequenceLast[i]+1))&0x3FFF);
					SetCtrlVal(ScanBurstPanel, SCANBURST_Text, Str);
				}
				else if (SequenceLast[i] == 0) {
					sprintf(Str, "%6d First ApID %03Xh %s\n", Total-1, ID, GetTime1(UTCs,T1));
					SetCtrlVal(ScanBurstPanel, SCANBURST_Text, Str);
				}
				SequenceLast[i] = Sequence;
				UTCsLast[i] = UTCs;
				PktNoLast[i] = Total-1;
			}
		}
	}
	for(i=0; i<4; i++) if((BurstID[i] == SBSel) || (SBSel == 0)) {
		sprintf(Str, "%6d                              Last ApID %03Xh %s\n", PktNoLast[i], BurstID[i], GetTime1(UTCsLast[i],T1));
		SetCtrlVal(ScanBurstPanel, SCANBURST_Text, Str);
	}
	SetCtrlVal(ScanBurstPanel, SCANBURST_209, PacketCount[0]);
	SetCtrlVal(ScanBurstPanel, SCANBURST_211, PacketCount[1]);
	SetCtrlVal(ScanBurstPanel, SCANBURST_221, PacketCount[2]);
	SetCtrlVal(ScanBurstPanel, SCANBURST_223, PacketCount[3]);
	SetCtrlVal(ScanBurstPanel, SCANBURST_Other, PacketCount[4]);
	SetCtrlVal(ScanBurstPanel, SCANBURST_Total, Total);
	
	SetWaitCursor(0);
}

int CVICALLBACK ScanBurst (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			ScanBurst1();
			break;
	}
	return 0;
}


int CVICALLBACK ScanBurstSelect (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			ScanBurst1();
			break;
		}
	return 0;
}

// Burst Timing scan

static unsigned char BurstSend[100000], BurstSave[100000], BurstCur[100000], BurstBest[100000];
static unsigned int  BurstUTC[100000];

int CVICALLBACK PlotBurst (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	unsigned int PacketCount, HKPCount, ID, UTCs, i;
	char Str[50];
	
	switch (event) {
		case EVENT_COMMIT:
			SetWaitCursor(1);
			DisplayPanel ( PlotBurstPanel );
			DeleteGraphPlot (PlotBurstPanel, PLOTBURST_GRAPH, -1,
                 VAL_IMMEDIATE_DRAW);
            ClearAxisItems (PlotBurstPanel, PLOTBURST_GRAPH, VAL_XAXIS );
            PacketCount=HKPCount=0;
			while(GetData() && HKPCount < 100000) {
				PacketCount++;
				if(((ID = SwapWord(Data, 0))&0x7FF) == 0x200) {
					SwapTelemetryBytes(Data);
					BurstUTC[HKPCount] = GetLong3412(32);
					BurstSend[HKPCount] = GetByte(26+70);
					BurstSave[HKPCount] = GetByte(26+69);
					BurstCur[HKPCount] = GetByte(26+59);
					BurstBest[HKPCount] = GetByte(26+71);
					HKPCount++;
				}
			}
			PlotXY (PlotBurstPanel, PLOTBURST_GRAPH, BurstUTC, BurstSend, HKPCount,
        		VAL_UNSIGNED_INTEGER, VAL_UNSIGNED_CHAR, VAL_THIN_LINE, VAL_NO_POINT,
        		VAL_SOLID, 1, VAL_RED);
			PlotXY (PlotBurstPanel, PLOTBURST_GRAPH, BurstUTC, BurstSave, HKPCount,
        		VAL_UNSIGNED_INTEGER, VAL_UNSIGNED_CHAR, VAL_THIN_LINE, VAL_NO_POINT,
        		VAL_SOLID, 1, VAL_BLUE);
			PlotXY (PlotBurstPanel, PLOTBURST_GRAPH, BurstUTC, BurstCur, HKPCount,
        		VAL_UNSIGNED_INTEGER, VAL_UNSIGNED_CHAR, VAL_THIN_LINE, VAL_NO_POINT,
        		VAL_SOLID, 1, VAL_DK_GREEN);
			PlotXY (PlotBurstPanel, PLOTBURST_GRAPH, BurstUTC, BurstBest, HKPCount,
        		VAL_UNSIGNED_INTEGER, VAL_UNSIGNED_CHAR, VAL_THIN_LINE, VAL_NO_POINT,
        		VAL_SOLID, 1, VAL_BLACK);

			for(i=0; i<HKPCount-1; i++) {
				if(BurstBest[i+1] < BurstBest[i]) 
					InsertAxisItem (PlotBurstPanel, PLOTBURST_GRAPH, VAL_XAXIS, -1,
						GetTime2(BurstUTC[i+1], Str), BurstUTC[i+1]);
			}
			InsertAxisItem (PlotBurstPanel, PLOTBURST_GRAPH, VAL_XAXIS, -1,
				GetTime1(BurstUTC[0], Str), BurstUTC[0]);
			InsertAxisItem (PlotBurstPanel, PLOTBURST_GRAPH, VAL_XAXIS, -1,
				GetTime2(BurstUTC[HKPCount-1], Str), BurstUTC[HKPCount-1]);

			SetWaitCursor(0);

			break;
	}
	return 0;
}

int CVICALLBACK PlotBurstActive (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char Str[100], ST[50];
	double X, Y;
	
	switch (event)
		{
		case EVENT_LEFT_CLICK:
			GetGraphCursor (PlotBurstPanel, PLOTBURST_GRAPH, 1, &X, &Y);
			sprintf(Str, "Cursor at: %s, %d", GetTime1((int)X, ST), (int)Y);
			ResetTextBox(PlotBurstPanel, PLOTBURST_CURSOR, Str);
			break;
		}
	return 0;
}

static unsigned char BC_SWEA[100000], BC_STE[100000], BC_MAG[100000], BC_PLASTIC[100000];
static unsigned char BC_NTDS[100000], BC_PLASMA[100000], BC_HFR1[100000], BC_HFR2[100000];

int CVICALLBACK PlotBurstCriteria (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i, j, Dat, NS, T0;
	float Val;
	char Str[50];
	static int Colors[8] = {
		VAL_BLACK, 0x803000, VAL_RED, 0xFF8000L, VAL_YELLOW,
		VAL_GREEN, VAL_BLUE, VAL_DK_MAGENTA};
	
	switch (event) {
		case EVENT_COMMIT:
			DisplayPanel ( PlotBurstCritPanel );
			DeleteGraphPlot (PlotBurstCritPanel, BURSTCRIT_GRAPH, -1,
                 VAL_IMMEDIATE_DRAW);
            ClearAxisItems (PlotBurstCritPanel, BURSTCRIT_GRAPH, VAL_XAXIS );
			SetWaitCursor(1);
			NS = 0;
			while(GetData() && (NS<SIZE)) if((SwapWord(Data, 0)&0x7FF) == 0x203) {
				SwapTelemetryBytes(Data);
				T0 = GetLong3412(32);
				for(i=0; i<32; i++) {
					BurstUTC[NS] = T0 + i;
					BC_SWEA[NS] = GetByte(26+15+8*i);
					BC_STE[NS] = GetByte(26+15+8*i+1);
					BC_MAG[NS] = GetByte(26+15+8*i+2);
					BC_PLASTIC[NS] = GetByte(26+15+8*i+3);
					BC_NTDS[NS] = GetByte(26+15+8*i+4);
					BC_PLASMA[NS] = GetByte(26+15+8*i+5);
					BC_HFR1[NS] = GetByte(26+15+8*i+6);
					BC_HFR2[NS] = GetByte(26+15+8*i+7);
					NS++;
				}
			}

			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_SWEA, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_SWEA, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[0]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_STE, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_STE, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[1]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_MAG, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_MAG, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[2]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_PLASTIC, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_PLASTIC, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[3]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_NTDS, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_NTDS, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[4]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_PLASMA, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_PLASMA, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[5]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_HFR1, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_HFR1, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[6]);
			GetCtrlVal(PlotBurstCritPanel, BURSTCRIT_HFR2, &i);
			if(i) PlotXY (PlotBurstCritPanel, BURSTCRIT_GRAPH, BurstUTC, BC_HFR2, NS,
        				VAL_INTEGER, VAL_UNSIGNED_CHAR, VAL_FAT_LINE, VAL_NO_POINT,
        				VAL_SOLID, 1, Colors[7]);

			InsertAxisItem (PlotBurstCritPanel, BURSTCRIT_GRAPH, VAL_XAXIS, -1,
				GetTime1(BurstUTC[0], Str), BurstUTC[0]);
			InsertAxisItem (PlotBurstCritPanel, BURSTCRIT_GRAPH, VAL_XAXIS, -1,
				GetTime1(BurstUTC[NS-1], Str), BurstUTC[NS-1]);
			
			SetWaitCursor(0);
			break;
	}
	return 0;
}


int CVICALLBACK Print (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			PrintPanel (panel, "", 1, VAL_FULL_PANEL, 1);
			break;
		}
	return 0;
}


