// MAGIFCFit.c
//   Routine to fit MAG data taken during the transition to/from IFC
//   to an exponetial response in order to determine the MAG AC performance
//
//  D.W.Curtis   2004-02-11

#include <userint.h>
#include "Decom.h"
#include <ansi_c.h>

unsigned int GetWord(int i);
int GetByte(int i);
long GetLong3412(int i);
int GetApID(void);
int PacketNumber;
float lsq(void);

float Y[1000], Sig[1000], P1[20];
int nY,nP;
int nF, P0[20];
float (*func[10])(float x, float P[]), (*dfunc[10])(int iX, int iP, float P[]);

static int XYZ;
char *Result;
static int MAGIFCPanel=-1;

static float FitFunc(float x, float P[])
{
	if(x > P[2]) return P[0]+P[1]*exp(-(x-P[2])/P[3])+P[4]*exp(-(x-P[2])/P[5]);
	else return P[0]+P[1]+P[4];
}

static float dFitFunc(int iX, int iP, float P[])
{
	if(iP == 0) return 1.F;
	if(iX > P[2]) {
		if(iP == 1) return exp(-(iX-P[2])/P[3]);
		if(iP == 2) return P[1]*exp(-(iX-P[2])/P[3])/P[3] + P[4]*exp(-(iX-P[2])/P[5])/P[5];
		if(iP == 3) return P[1]*exp(-(iX-P[2])/P[3])*(iX-P[2])/(P[3]*P[3]);
		if(iP == 4) return exp(-(iX-P[2])/P[5]);
		return P[4]*exp(-(iX-P[2])/P[5])*(iX-P[2])/(P[5]*P[5]);
	}
	if((iP == 1) || (iP == 4)) return 1.F;
	return 0.F;
}

static char *GetT(float offset)
{
	long T;
	static char dtime[45],atime[26],fsec[20];
	float sec;
	extern long FIFTYEIGHT;

	T = GetLong3412(32)+FIFTYEIGHT;
	sec = GetByte(37)/256.+GetByte(36)/65536.+offset;
	while(sec >=1.) { sec -= 1.; T++; }
	strcpy(atime,asctime(gmtime(&T)));
	atime[5]-=0x20;		//Captialize
	atime[6]-=0x20;		//Captialize
	memcpy(dtime,atime+20,4); //YYYY
	memcpy(dtime+4,atime+3,4); // MMM
	memcpy(dtime+8,atime+7,3); // DD
	memcpy(dtime+11,atime+10,9); // HH:MM:SS
	sprintf(fsec,"%5.4f",sec);
	memcpy(dtime+20,fsec+1,7);
	dtime[27]='\0';
	return dtime;
}

static void FitIFC()
{
	int i, T0;
	float Chisq, X[1000], R, D, T1, T2;
	
	nY = 42;
	nP = 6;
	nF = 1;
	P0[0] = 0;
	func[0] = FitFunc;
	dfunc[0] = dFitFunc;
	for(i=0; i<nY; i++) Sig[i] = 1.F;
	DeleteGraphPlot (MAGIFCPanel, MAGIFC_GRAPH, -1, VAL_IMMEDIATE_DRAW);
	SetCtrlVal(MAGIFCPanel, MAGIFC_PacketNum, PacketNumber);
	T0 = GetByte(26+12);

	R = 0.003;
	D = 0.7;
	T1 = .3;
	T2 = 2.;
// X
	Result = "OK";
	for(i=0; i<nY; i++) Y[i] = (float)((int)GetWord(26+18+6*i)-32768);
	if(Y[0] < Y[nY-1]) {
		for(i=1; (i<nY) && (Y[i] < (Y[0]+Y[nY-1])/2); i++);
	}
	else {
		for(i=1; (i<nY) && (Y[i] > (Y[0]+Y[nY-1])/2); i++);
	}
	
	P1[0] = Y[nY-1];
	P1[1] = (Y[0]-Y[nY-1]);
	P1[2] = (float)i-D;
	P1[3] = T1;
	P1[4] = P1[1]*R;
	P1[1] = P1[1]*(1.-R);
	P1[5] = T2;
	Chisq = lsq();
	SetCtrlVal(MAGIFCPanel, MAGIFC_OffsetX, P1[0]);
	SetCtrlVal(MAGIFCPanel, MAGIFC_DeltaX, -(P1[1]+P1[4]));
	SetCtrlVal(MAGIFCPanel, MAGIFC_TimeX, (P1[2]+T0-32)*1000./32.);
	SetCtrlVal(MAGIFCPanel, MAGIFC_RateX, P1[3]*1000./32.);
	SetCtrlVal(MAGIFCPanel, MAGIFC_MsgX, Result);
	SetCtrlVal(MAGIFCPanel, MAGIFC_UTCX, GetT(P1[2]/32.));
	SetCtrlVal(MAGIFCPanel, MAGIFC_A2X, P1[4]/P1[1]);
	SetCtrlVal(MAGIFCPanel, MAGIFC_R2X, P1[5]*1000./32.);
	for(i=0; i<nY; i++) {
		Y[i] = (Y[i]-P1[0])/(P1[1]+P1[4]);
		X[i] = (i-P1[2])*1000./32.;
	}
	PlotXY (MAGIFCPanel, MAGIFC_GRAPH, X, Y, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
	for(i=0; i<nY*10; i++) {
		Y[i] = (FitFunc((float)i/10., P1)-P1[0])/(P1[1]+P1[4]);
		X[i] = (i/10.-P1[2])*1000./32.;
	}
	PlotXY (MAGIFCPanel, MAGIFC_GRAPH, X, Y, nY*10, VAL_FLOAT, VAL_FLOAT, VAL_THIN_LINE,
       				VAL_NO_POINT, VAL_SOLID, 1, VAL_RED);

// Y
	Result = "OK";
	for(i=0; i<nY; i++) Y[i] = (float)((int)GetWord(26+18+6*i+2)-32768);
	if(Y[0] < Y[nY-1]) {
		for(i=1; (i<nY) && (Y[i] < (Y[0]+Y[nY-1])/2); i++);
	}
	else {
		for(i=1; (i<nY) && (Y[i] > (Y[0]+Y[nY-1])/2); i++);
	}
	
	P1[0] = Y[nY-1];
	P1[1] = (Y[0]-Y[nY-1]);
	P1[2] = (float)i-D;
	P1[3] = T1;
	P1[4] = P1[1]*R;
	P1[1] = P1[1]*(1.-R);
	P1[5] = T2;
	Chisq = lsq();
	SetCtrlVal(MAGIFCPanel, MAGIFC_OffsetY, P1[0]);
	SetCtrlVal(MAGIFCPanel, MAGIFC_DeltaY, -(P1[1]+P1[4]));
	SetCtrlVal(MAGIFCPanel, MAGIFC_TimeY, (P1[2]+T0-32)*1000./32.);
	SetCtrlVal(MAGIFCPanel, MAGIFC_RateY, P1[3]*1000./32.);
	SetCtrlVal(MAGIFCPanel, MAGIFC_MsgY, Result);
	SetCtrlVal(MAGIFCPanel, MAGIFC_UTCY, GetT(P1[2]/32.));
	SetCtrlVal(MAGIFCPanel, MAGIFC_A2Y, P1[4]/P1[1]);
	SetCtrlVal(MAGIFCPanel, MAGIFC_R2Y, P1[5]*1000./32.);
	for(i=0; i<nY; i++) {
		Y[i] = (Y[i]-P1[0])/P1[1];
		X[i] = (i-P1[2])*1000./32.;
	}
	PlotXY (MAGIFCPanel, MAGIFC_GRAPH, X, Y, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_DK_GREEN);
	for(i=0; i<nY*10; i++) {
		Y[i] = (FitFunc((float)i/10., P1)-P1[0])/(P1[1]+P1[4]);
		X[i] = (i/10.-P1[2])*1000./32.;
	}
	PlotXY (MAGIFCPanel, MAGIFC_GRAPH, X, Y, nY*10, VAL_FLOAT, VAL_FLOAT, VAL_THIN_LINE,
       				VAL_NO_POINT, VAL_SOLID, 1, VAL_DK_GREEN);


// Z
	Result = "OK";
	for(i=0; i<nY; i++) Y[i] = (float)((int)GetWord(26+18+6*i+4)-32768);
	if(Y[0] < Y[nY-1]) {
		for(i=1; (i<nY) && (Y[i] < (Y[0]+Y[nY-1])/2); i++);
	}
	else {
		for(i=1; (i<nY) && (Y[i] > (Y[0]+Y[nY-1])/2); i++);
	}

	P1[0] = Y[nY-1];
	P1[1] = (Y[0]-Y[nY-1]);
	P1[2] = (float)i-D;
	P1[3] = T1;
	P1[4] = P1[1]*R;
	P1[1] = P1[1]*(1.-R);
	P1[5] = T2;
	Chisq = lsq();
	SetCtrlVal(MAGIFCPanel, MAGIFC_OffsetZ, P1[0]);
	SetCtrlVal(MAGIFCPanel, MAGIFC_DeltaZ, -(P1[1]+P1[4]));
	SetCtrlVal(MAGIFCPanel, MAGIFC_TimeZ, (P1[2]+T0-32)*1000./32.);
	SetCtrlVal(MAGIFCPanel, MAGIFC_RateZ, P1[3]*1000./32.);
	SetCtrlVal(MAGIFCPanel, MAGIFC_MsgZ, Result);
	SetCtrlVal(MAGIFCPanel, MAGIFC_UTCZ, GetT(P1[2]/32.));
	SetCtrlVal(MAGIFCPanel, MAGIFC_A2Z, P1[4]/P1[1]);
	SetCtrlVal(MAGIFCPanel, MAGIFC_R2Z, P1[5]*1000./32.);
	for(i=0; i<nY; i++) {
		Y[i] = (Y[i]-P1[0])/P1[1];
		X[i] = (i-P1[2])*1000./32.;
	}
	PlotXY (MAGIFCPanel, MAGIFC_GRAPH, X, Y, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_DK_BLUE);
	for(i=0; i<nY*10; i++) {
		Y[i] = (FitFunc((float)i/10., P1)-P1[0])/(P1[1]+P1[4]);
		X[i] = (i/10.-P1[2])*1000./32.;
	}
	PlotXY (MAGIFCPanel, MAGIFC_GRAPH, X, Y, nY*10, VAL_FLOAT, VAL_FLOAT, VAL_THIN_LINE,
       				VAL_NO_POINT, VAL_SOLID, 1, VAL_DK_BLUE);
	
}


int CVICALLBACK MAGIFCFit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			if((GetApID()&0xFFE) != 0x208) break;
			if(MAGIFCPanel == -1) MAGIFCPanel = LoadPanel(0, "DECOM.uir", MAGIFC);
			DisplayPanel(MAGIFCPanel);
			FitIFC();
			break;
		}
	return 0;
}

void err_out(char *Error)
{
	Result = Error;
}

