/*  MATRIX.C

	Matrix functions:
		det = smatinv(A,N)		Matrix inversion
			A = NxN symetric matrix to invert
			det is the determinant (0. -> singular)

		mvmult(R,V,A,N)			Matrix - vector multiply
			A = R x V;
			R = NxN matrix
			A,V = N vectors

	(c) Copyright  D.W.Curtis  1987,95
	Converted to MSC  10-95  DWC
*/


static float ONE = 1.F, ZERO = 0.F;

#define	abs(X)	(X < ZERO ? -X : X)
#define NMAX 20

float smatinv(float R[], int N)
{
	float det,amax,save;
	int i,j,k,l,l1,l2;
	int ikk,jkk;
	int ik[NMAX], jk[NMAX];

	det = ONE;
	for(k=0; k<N; k++) {
		amax = ZERO;			/* Find largest remaining element */
		for(i=k; i<N; i++) {
			l1 = i*N + k;
			for(j=k; j<N; j++) {
				if(abs(amax) <= abs(R[l1])) {
					amax = R[l1];
					ik[k] = ikk = i;
					jk[k] = jkk = j;
				}
				l1++;
			}
		}
		if(amax == ZERO) return(ZERO);

/* Now swap rows,cols to get amax to R[k,k] */
		if(ikk > k) {
			l1 = ikk*N;
			l2 = k*N;
			for(j=0; j<N; j++) {
				save = R[l2];
				R[l2++] = R[l1];
				R[l1++] = -save;
			}
		}
		if(jkk > k) for(i=0; i<N; i++) {
			l1 = i*N;
			save = R[l1+k];
			R[l1+k] = R[l1+jkk];
			R[l1+jkk] = -save;
		}

/* Accumulate elements of inverse matrix */
		for(i=0; i<N; i++) if(i != k) R[i*N+k] /= -amax;
		l2 = k*N;
		for(i=0; i<N; i++) if(i != k) {
			l1 = i*N;
			for(j=0; j<N; j++) if(j != k)
				R[l1+j] += R[l1+k]*R[l2+j];
		}
		for(j=0; j<N; j++) if(j != k) R[l2+j] /= amax;
		R[l2+k] = ONE/amax;
		det *= amax;
	}

/* Restore ordering of matrix */

	for(l=0; l<N; l++) {
		k = N-l-1;
		if((j = ik[k]) > k) for(i=0; i<N; i++) {
			l1 = i*N;
			save = R[l1+k];
			R[l1+k] = -R[l1+j];
			R[l1+j] = save;
		}
		if((i = jk[k]) > k) {
			l1 = i*N;
			l2 = k*N;
			for(j=0; j<N; j++) {
				save = R[l2];
				R[l2++] = -R[l1];
				R[l1++] = save;
			}
		}
	}

	return(det);
}

void mvmult(float *R, float V[], float *A, int N)
{
	int i,j;
	float sum;

	for(i=0; i<N; i++) {
		sum=ZERO;
		for(j=0; j<N; j++) sum += V[j] * *R++;
		*A++ = sum;
	}
}
