// STE
//   Display STE monitor rates & accumulations

#include <stdio.h>
#include <userint.h>
#include <ansi_c.h>
#include "DECOM.h"

extern int PlotSTERatesPanel, PlotSTESpecPanel, MainPanel;
extern unsigned char Data[];
int SwapWord(unsigned char *Data, int X); 
void SwapTelemetryBytes(char *Data);
int GetByte(int i);
unsigned int GetWord(int i);
long GetLong3412(int i);
char *GetTime1(unsigned int UTCs, char *dtime);
char *GetTime2(unsigned int UTCs, char *dtime);
char *GetTimeX(unsigned int UTCs, char *dtime);
int GetData(void);
static int Colors[24] = {
	VAL_BLACK, 0x803000, VAL_RED, 0xFF8000L, VAL_YELLOW,
	VAL_GREEN, VAL_BLUE, VAL_DK_MAGENTA, VAL_GRAY, VAL_WHITE,
	VAL_BLACK, VAL_BLACK, VAL_BLACK, VAL_BLACK,
	VAL_BLACK, VAL_BLACK, VAL_BLACK, VAL_BLACK,
	VAL_BLACK, VAL_BLACK, VAL_BLACK, VAL_BLACK,
	VAL_BLACK, VAL_BLACK
};

#define SIZE 1000000

int CVICALLBACK PlotSTERates (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	extern float Samples[24][SIZE];
	extern long UTC[SIZE];
	int NS, Interval, i, j, AS, LOG, Dat, SEL, Col;
	long UTC0;
	double Min, Max;
	char Str[100];
	static int Sel[24] = {
		STERATES_LLD0, STERATES_LLD1, STERATES_LLD2, STERATES_LLD3,
		STERATES_ULD0, STERATES_ULD1, STERATES_ULD2, STERATES_ULD3,
		STERATES_RST0, STERATES_RST1, STERATES_RST2, STERATES_RST3,
		STERATES_LLD4, STERATES_LLD5, STERATES_LLD6, STERATES_LLD7,
		STERATES_ULD4, STERATES_ULD5, STERATES_ULD6, STERATES_ULD7,
		STERATES_RST4, STERATES_RST5, STERATES_RST6, STERATES_RST7
	};		

	switch (event) {
		case EVENT_COMMIT:
			DisplayPanel ( PlotSTERatesPanel );
			DeleteGraphPlot (PlotSTERatesPanel, STERATES_GRAPH, -1,
                 VAL_IMMEDIATE_DRAW);
            ClearAxisItems (PlotSTERatesPanel, STERATES_GRAPH, VAL_XAXIS );
			SetWaitCursor(1);
			NS=0;
			while(GetData() && (NS<SIZE)) if((SwapWord(Data, 0)&0x7FF) == 0x220) {
				SwapTelemetryBytes(Data);
				UTC0 = GetLong3412(32);
				Interval = GetByte(26+12);
				for(i=0; i<10; i++) {	// 10 samples per packet
					for(j=0; j<24; j++) {	// 24 rates
						Dat = GetByte(26+21+25*i+j);
						Samples[j][NS] = ((float)((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1)))/Interval;
					}
					UTC[NS++] = UTC0;
					UTC0 += Interval;
				}
			}

			GetCtrlVal(PlotSTERatesPanel, STERATES_AutoScale, &AS);
			if(AS) {
				Min = 1.E6;
				Max = 1.;
				for(j=0; j<24; j++) {
					GetCtrlVal(PlotSTERatesPanel, Sel[j], &SEL);
					if(SEL) for(i=0; i<NS; i++) {
						if (Samples[j][i] < Min) Min = Samples[j][i];
						if (Samples[j][i] > Max) Max = Samples[j][i];
					}
					if(Min < 1.) Min = 1.;
				}
				SetCtrlVal(PlotSTERatesPanel, STERATES_MIN, Min);
				SetCtrlVal(PlotSTERatesPanel, STERATES_MAX, Max);
			}
			else {
				GetCtrlVal(PlotSTERatesPanel, STERATES_MIN, &Min);
				GetCtrlVal(PlotSTERatesPanel, STERATES_MAX, &Max);
			}
			GetCtrlVal(PlotSTERatesPanel, STERATES_Log, &LOG); 
			SetCtrlAttribute (PlotSTERatesPanel, STERATES_GRAPH,
				ATTR_YMAP_MODE, LOG?VAL_LOG:VAL_LINEAR);			
			SetAxisScalingMode (PlotSTERatesPanel, STERATES_GRAPH,
                    VAL_LEFT_YAXIS, VAL_MANUAL, Min, Max);
                    
			for(i=Col=0; i<24; i++) {
				GetCtrlVal(PlotSTERatesPanel, Sel[i], &SEL);
       			SetCtrlAttribute(PlotSTERatesPanel, Sel[i], ATTR_ON_COLOR,
       				SEL?Colors[Col]:VAL_WHITE);
				if(SEL)
				PlotXY (PlotSTERatesPanel, STERATES_GRAPH, UTC, Samples[i], NS,
       				VAL_UNSIGNED_INTEGER, VAL_FLOAT, VAL_THIN_LINE, VAL_NO_POINT,
       				VAL_SOLID, 1, Colors[Col++]);
			}
			InsertAxisItem (PlotSTERatesPanel, STERATES_GRAPH, VAL_XAXIS, -1,
				GetTime1(UTC[0], Str), UTC[0]);
			InsertAxisItem (PlotSTERatesPanel, STERATES_GRAPH, VAL_XAXIS, -1,
				GetTime1(UTC[NS-1], Str), UTC[NS-1]);
			
			SetWaitCursor(0);
			break;
		}

	return 0;
}


static long UTCLast=0;
double Spectra[512];
int AccTime;
int NumEnergy0, Mode0;

int CVICALLBACK AccSTESpec (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	double Min, Max, Spec[512];
	int i, NS, Interval, End, AS, LOG, SD, RATES;
	int NumEnergy, Mode, Start, Dat;
	long UTC, UTC0;
	char Str[100];
	
	switch (event)
		{
		case EVENT_COMMIT:
			DisplayPanel ( PlotSTESpecPanel );
			DeleteGraphPlot (PlotSTESpecPanel, STESPEC_GRAPH, -1,
                 VAL_IMMEDIATE_DRAW);
			SetWaitCursor(1);
			NS=AccTime=End=0;
			for(i=0; i<512; i++) Spectra[i] = 0.;
			UTCLast=0;
			
			while(GetData()) if((SwapWord(Data, 0)&0x7FF) == 0x222) {
				SwapTelemetryBytes(Data);
				Interval = GetByte(26+10);
				NumEnergy = GetByte(26+12);
				Mode = GetByte(26+13);
				Start = GetByte(26+14);
				if(NS == 0) {
					Mode0 = Mode;
					SetCtrlVal(PlotSTESpecPanel, STESPEC_MODE, Mode);
					NumEnergy0 = NumEnergy;
					UTC0 = GetLong3412(32);
					GetTime1(UTC0, Str);
					ResetTextBox(PlotSTESpecPanel, STESPEC_UTCStart, Str);
				}
				else if((Mode != Mode0) || (NumEnergy != NumEnergy0)) {
					GetTime1(UTCLast=UTC, Str);
					ResetTextBox(PlotSTESpecPanel, STESPEC_UTCEnd, Str);
					End = 1;
				}
				if(!End) {
					UTC = GetLong3412(32);
					if(NumEnergy == 16) {	// 2 spectra per packet
						for(i=0; i<128; i++) {
							Dat = GetByte(26+15+i);
							Spectra[i] += ((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1));
						}
						AccTime += Interval;
						NS++;
						for(i=0; i<128; i++) {
							Dat = GetByte(26+15+128+i);
							Spectra[i] += ((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1));
						}
						AccTime += Interval;
						NS++;
					}
					else if(NumEnergy == 64) {	// 2 packets per accum.
						if(Start == 0) {
							for(i=0; i<256; i++) {
								Dat = GetByte(26+15+i);
								Spectra[i] += ((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1));
							}
						}
						else {
							for(i=0; i<256; i++) {
								Dat = GetByte(26+15+i);
								Spectra[i+256] += ((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1));
							}
							AccTime += Interval;
							NS++;
						}
					}
					else {	// assume 32E default
						for(i=0; i<256; i++) {
							Dat = GetByte(26+15+i);
							Spectra[i] += ((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1));
						}
						AccTime += Interval;
						NS++;
					}
				}
			}
			if(!End) {
				GetTime1(UTC, Str);
				ResetTextBox(PlotSTESpecPanel, STESPEC_UTCEnd, Str);
			}
			
			SetCtrlVal(PlotSTESpecPanel, STESPEC_ACCTIME, AccTime);
			
			GetCtrlVal(PlotSTESpecPanel, STESPEC_RATES, &RATES);
			if(RATES) {
				for(i=0; i<512; i++) Spec[i] = Spectra[i] / AccTime;
				SetCtrlAttribute (PlotSTESpecPanel, STESPEC_GRAPH, ATTR_YNAME, "Counts/Sec");
			}
			else {
				for(i=0; i<512; i++) Spec[i] = Spectra[i];
				SetCtrlAttribute (PlotSTESpecPanel, STESPEC_GRAPH, ATTR_YNAME, "Counts");
			}
			
			
			GetCtrlVal(PlotSTESpecPanel, STESPEC_AutoScale, &AS);
			if(AS) {
				Min = 1.E6;
				Max = 0.;
				for(i=0; i<512; i++) {
					if ((Spec[i] < Min) && (Spec[i] != 0.)) Min = Spec[i];
					if (Spec[i] > Max) Max = Spec[i];
				}
				SetCtrlVal(PlotSTESpecPanel, STESPEC_MIN, Min);
				SetCtrlVal(PlotSTESpecPanel, STESPEC_MAX, Max);
			}
			else {
				GetCtrlVal(PlotSTESpecPanel, STESPEC_MIN, &Min);
				GetCtrlVal(PlotSTESpecPanel, STESPEC_MAX, &Max);
			}
			GetCtrlVal(PlotSTESpecPanel, STESPEC_Log, &LOG); 
			SetCtrlAttribute (PlotSTESpecPanel, STESPEC_GRAPH,
				ATTR_YMAP_MODE, LOG?VAL_LOG:VAL_LINEAR);			
			SetAxisScalingMode (PlotSTESpecPanel, STESPEC_GRAPH,
                    VAL_LEFT_YAXIS, VAL_MANUAL, Min, Max);
			GetCtrlVal(PlotSTESpecPanel, STESPEC_SingleDet, &SD); 

            if(SD) {
				PlotY (PlotSTESpecPanel, STESPEC_GRAPH, Spec, NumEnergy0*4,
       				VAL_DOUBLE, VAL_THIN_LINE, VAL_NO_POINT,
       				VAL_SOLID, 1, Colors[0]);
				PlotY (PlotSTESpecPanel, STESPEC_GRAPH, &Spec[NumEnergy0*4], NumEnergy0*4,
       				VAL_DOUBLE, VAL_THIN_LINE, VAL_NO_POINT,
       				VAL_SOLID, 1, Colors[4]);
            }
            else for(i=0; i<8; i++) {
				PlotY (PlotSTESpecPanel, STESPEC_GRAPH, &Spec[NumEnergy*i], NumEnergy0,
       				VAL_DOUBLE, VAL_THIN_LINE, VAL_NO_POINT,
       				VAL_SOLID, 1, Colors[i]);
			}
			
			SetWaitCursor(0);
			break;
		}
	return 0;
}

int CVICALLBACK STEAccNext (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char Str[100];
	
	switch (event)
		{
		case EVENT_COMMIT:
			if(UTCLast) {
				GetTime1(UTCLast+120, Str);
				ResetTextBox(MainPanel, DECOM_UTCStart, Str);
				SetCtrlVal(MainPanel, DECOM_FullRange, 0);
				AccSTESpec (panel, control, event, callbackData, eventData1, eventData2);
			}
			break;
		}
	return 0;
}

int CVICALLBACK SaveSTEAcc (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	FILE *File;
	extern char FileName[];
	char OutName[100], Str[100];
	int i, SD;
	
	
	switch (event) {
		case EVENT_COMMIT:
			strcpy(OutName, FileName);
			strcpy(strrchr(OutName, '.'), "STE.txt");
			File = fopen(OutName, "w");
			fprintf(File, "STEREO IMPACT STE Spectra Data\n");
			fprintf(File, " Data extracted from telemetry file %s\n", FileName);

			GetCtrlVal(PlotSTESpecPanel, STESPEC_UTCStart, Str);
			fprintf(File, "  Accumulated over time interval %s", Str);
			GetCtrlVal(PlotSTESpecPanel, STESPEC_UTCEnd, Str);
			fprintf(File, " - %s, %d seconds, Mode %d\n\n", Str, AccTime, Mode0);
			
			GetCtrlVal(PlotSTESpecPanel, STESPEC_SingleDet, &SD); 
			
			if(SD) {
				fprintf(File, "Bin\tSTE-U\tSTE-D\n");
				for(i=0; i<NumEnergy0*4; i++)
					fprintf(File, "%3d\t%7.0f\t%7.0f\n", i, Spectra[i], Spectra[i+4*NumEnergy0]);
			}
			else {
				fprintf(File, "Bin\tSTE-U-0\tSTE-U-1\tSTE-U-2\tSTE-U-3\tSTE-D-0\tSTE-D-1\tSTE-D-2\tSTE-D-3\n");
				for(i=0; i<NumEnergy0; i++)
					fprintf(File, "%3d\t%7.0f\t%7.0f\t%7.0f\t%7.0f\t%7.0f\t%7.0f\t%7.0f\t%7.0f\n", i,
						Spectra[i], Spectra[i+NumEnergy0], Spectra[i+2*NumEnergy0], Spectra[i+3*NumEnergy0],
						Spectra[i+4*NumEnergy0], Spectra[i+5*NumEnergy0], Spectra[i+6*NumEnergy0], Spectra[i+7*NumEnergy0]);
			}
			fclose(File);
			break;
	}
			
	return 0;
}
