//  STEFit
//   Fit STE spectra to known door source lines
//
//  D.W.Curtis 2004-5-19


#include <userint.h>
#include "Decom.h"
#include <ansi_c.h>

extern int PlotSTESpecPanel;
static int STEFitPanel=-1;
static int STEFitPPanel=-1;
unsigned int GetWord(int i);
int GetByte(int i);
long GetLong3412(int i);
int GetApID(void);
int PacketNumber;
float lsq(void);
extern double Spectra[];
extern int AccTime; 
extern int NumEnergy0;

extern float Y[], Sig[];
extern int nY,nP;
extern int nF, P0[];
extern float P1[];
extern float (*func[10])(float x, float P[]), (*dfunc[10])(int iX, int iP, float P[]);

#define REGIONS 3
#define PEAKS 4
#define SIGMA (REGIONS+PEAKS)
#define ENERGY0 (REGIONS+PEAKS+1)
#define ENERGY1 (REGIONS+PEAKS+2)
#define ENERGY2 (REGIONS+PEAKS+3)

static int LowerLimit[REGIONS] = { 10, 49, 216 };
static int UpperLimit[REGIONS] = { 19, 71, 232 };
static int LowerLimitDL[REGIONS] = { 10, 21, 46 };
static int UpperLimitDL[REGIONS] = { 19, 43, 62 };
static int RegionLimits[REGIONS+1];
static float Energy[PEAKS] = { 5.895F, 22.08F, 25.0F, 88.04F };
static int PeakRegion[PEAKS] = { 0, 1, 1, 2 };
static int Scale;

extern char *Result;

static float gaus(float x, float bin, float amp, float sig)	/* compute gaussianlocated at bin, amplitude amp */
															/* sigma sig, for value x */
{
	if((x = (x-bin)/sig) > 5.) return(0.F);
	if(x < -5.) return(0.F);

	return((float) (amp*exp((double)-x*x/2.F)));
}


// Fit parameters:
//  i=0 - REGIONS-1 = baseline for region i
//  i=REGIONS - REGIONS+PEAKS-1 = amplitude for peak i-REGIONS
//  i=SIGMA = sigma for peaks
//  i=ENERGY0 = energy offset
//  i=ENERGY1 = energy gain
//  i=ENERGY2 = energy quadratic term (E = ENERGY0+ENERGY1*x*(1+ENERGY2*x))
static float Fit(float x, float PP[])	/* compute fit value */
{
	float F, x1, E, bin, C;            
	int i, Region;

	if(nP == ENERGY2) C = P1[ENERGY2];
	else C = PP[ENERGY2];
	
	for(i=0; i<REGIONS; i++) if((x >= RegionLimits[i]) && (x < RegionLimits[i+1])) Region = i;
	bin = (x + 0.5 + (LowerLimit[Region]/Scale) - RegionLimits[Region])*Scale;
	E = PP[ENERGY0]+bin*PP[ENERGY1]*(1.F+bin*C);

	F=PP[Region];
	for(i=0; i<PEAKS; i++) {
		if(PeakRegion[i] == Region) {
			F += gaus(E, Energy[i], PP[REGIONS+i], PP[SIGMA]);
		}
	}
	return F;
}


static float dFit(int iX, int iP, float PP[])
{
	int i, Region;
	float dF, bin, E, C;
	
	if(nP == ENERGY2) C = P1[ENERGY2];
	else C = PP[ENERGY2];
	
	for(i=Region=0; i<REGIONS; i++) if((iX >= RegionLimits[i]) && (iX < RegionLimits[i+1])) Region = i;
	bin = (float)(iX +0.5 + (LowerLimit[Region]/Scale) - RegionLimits[Region])*Scale;
	E = PP[ENERGY0]+bin*PP[ENERGY1]*(1.F+bin*C);

	if(iP < REGIONS) {	// baseline
		if(iP == Region) return 1.F;
		else return 0.F;
	}
	
	if(iP < REGIONS+PEAKS) {	//amplitude
		if(PeakRegion[iP-REGIONS] != Region) return 0.F;
		return gaus(E, Energy[iP-REGIONS], 1.F, PP[SIGMA]);
	}
	
	if(iP == SIGMA) {	// sigma
		for(i=0, dF=0.F; i<PEAKS; i++) if(PeakRegion[i] == Region) {
			dF += gaus(E, Energy[i], PP[REGIONS+i], PP[SIGMA])
				* (E-Energy[i]) * (E-Energy[i]) / 
				(PP[SIGMA]*PP[SIGMA]*PP[SIGMA]);
		}
		return dF;
	}
	
	if(iP == ENERGY0) {	// energy offset
		for(i=0, dF=0.F; i<PEAKS; i++) if(PeakRegion[i] == Region) {
			dF -= gaus(E, Energy[i], PP[REGIONS+i], PP[SIGMA])
				* (E-Energy[i]) / (PP[SIGMA]*PP[SIGMA]);
		}
		return dF;
	}
	
	if(iP == ENERGY1) {	// energy linear term
		for(i=0, dF=0.F; i<PEAKS; i++) if(PeakRegion[i] == Region) {
			dF -= gaus(E, Energy[i], PP[REGIONS+i], PP[SIGMA])
				* (E-Energy[i]) * bin * (1.F+bin*C) / (PP[SIGMA]*PP[SIGMA]);
		}
		return dF;
	}
	
	if(iP == ENERGY2) {	//energy quadratic term
		for(i=0, dF=0.F; i<PEAKS; i++) if(PeakRegion[i] == Region) {
			dF -= gaus(E, Energy[i], PP[REGIONS+i], PP[SIGMA])
				* (E-Energy[i]) * PP[ENERGY1]* bin * bin / (PP[SIGMA]*PP[SIGMA]);
		}
	}
	return dF;
}

#define SIG2FWHM	2.35482F	/* scale factor to convert sigma to FWHM */
#define G   2.5066F				/* scale factor from Amplitude*Sigma to integrated counts */

static void FitSTE()
{
	int i, j, k, RATES, SELECT, SINGLE, DETECTOR, CURVATURE, DOORLUT, j0;
	float Chisq, Sigma;
	float X[256], E[256], F[256];
	static float Rate0[PEAKS] = { 15.F, 10.F, 2.F, .003F };
	static float Base0[REGIONS] = {  2.F, 1.F,  0.001F };
	
	nP = REGIONS+PEAKS+4;
	nF = 1;
	P0[0] = 0;
	func[0] = Fit;
	dfunc[0] = dFit;
	
	GetCtrlVal(STEFitPanel, STEFIT_Select, &SELECT);
	j0 = SELECT?(NumEnergy0*4):0;
	GetCtrlVal(PlotSTESpecPanel, STESPEC_SingleDet, &SINGLE);
	GetCtrlVal(STEFitPanel, STEFIT_Detector, &DETECTOR);
	GetCtrlVal(STEFitPanel, STEFIT_FitCurvature, &CURVATURE);
	GetCtrlVal(STEFitPanel, STEFIT_DoorLUT, &DOORLUT);
	if(!SINGLE) j0 += DETECTOR*NumEnergy0;
	
	if(!CURVATURE) nP--;
	
	Scale = 256/NumEnergy0;
	if(SINGLE  || DOORLUT) Scale /= 4;

	RegionLimits[0] = 0;
	
	for(i=nY=0; i<REGIONS; i++) {
		if(DOORLUT) for(j=(LowerLimitDL[i]/Scale),k=0; j<=((UpperLimitDL[i]+Scale-1)/Scale); j++,k++)
			Y[nY++]=Spectra[j+j0];
		else for(j=(LowerLimit[i]/Scale),k=0; j<=((UpperLimit[i]+Scale-1)/Scale); j++,k++)
			Y[nY++]=Spectra[j+j0];
		RegionLimits[i+1] = RegionLimits[i]+k;
	}
	
	for(i=0; i<nY; i++) Sig[i] = pow((double)Y[i]+3., 1.5);	//Sig[i] = sqrt((double)Y[i])+1.F;
	
	Result = "OK";
	
	for(i=0; i<REGIONS; i++) P1[i] = Base0[i]*AccTime*Scale;	// baseline
	for(i=0; i<PEAKS; i++) P1[REGIONS+i] = Rate0[i]*AccTime*Scale;	// amplitude
	P1[SIGMA] = 1.0F;	//sigma
	P1[ENERGY0] = 0.F;
	P1[ENERGY1] = 0.39F;
	GetCtrlVal(STEFitPanel, STEFIT_QUAD, &P1[ENERGY2]);

	Chisq = lsq();

	SetCtrlVal(STEFitPanel, STEFIT_ChiSquared, Chisq);
	ResetTextBox(STEFitPanel, STEFIT_Msg, Result);
	SetCtrlVal(STEFitPanel, STEFIT_Baseline0, P1[0]/(AccTime*Scale));
	SetCtrlVal(STEFitPanel, STEFIT_Baseline1, P1[1]/(AccTime*Scale));
	SetCtrlVal(STEFitPanel, STEFIT_Baseline2, P1[2]/(AccTime*Scale));
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude0, P1[3]*P1[SIGMA]*G/(AccTime*P1[ENERGY1]*Scale));
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude1, P1[4]*P1[SIGMA]*G/(AccTime*P1[ENERGY1]*Scale));
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude2, P1[5]*P1[SIGMA]*G/(AccTime*P1[ENERGY1]*Scale));
	SetCtrlVal(STEFitPanel, STEFIT_Amplitude3, P1[6]*P1[SIGMA]*G/(AccTime*P1[ENERGY1]*Scale));
	Sigma = P1[SIGMA]*P1[SIGMA]-P1[ENERGY1]*P1[ENERGY1]/12.;	// deconvole bin width
	if(Sigma < 0.) Sigma = 0.;
	Sigma = sqrt((double)Sigma);
	SetCtrlVal(STEFitPanel, STEFIT_FWHM0, Sigma*SIG2FWHM);
	SetCtrlVal(STEFitPanel, STEFIT_OFFSET, P1[ENERGY0]);
	SetCtrlVal(STEFitPanel, STEFIT_GAIN, P1[ENERGY1]);
	SetCtrlVal(STEFitPanel, STEFIT_QUAD, P1[ENERGY2]);
	
	GetCtrlVal(PlotSTESpecPanel, STESPEC_RATES, &RATES);

	for(j=0; j<REGIONS; j++) {
		for(nY=0,i=RegionLimits[j]; i<RegionLimits[j+1]; i++) {
			F[nY] = Fit((float)i, P1);
			E[nY] = (F[nY]-Y[i])*(F[nY]-Y[i])/Sig[i];
			if(RATES) F[nY] /= AccTime;
			if(DOORLUT) X[nY++] = (float)(i-RegionLimits[j]+LowerLimitDL[j]/Scale);
			else X[nY++] = (float)(i-RegionLimits[j]+LowerLimit[j]/Scale);
		}
		PlotXY (PlotSTESpecPanel, STESPEC_GRAPH, X, F, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
//		PlotXY (PlotSTESpecPanel, STESPEC_GRAPH, X, E, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
//       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_BLUE);
    }
    
}

#define SIG 7
#define T 8
#define E0 9
#define E1 10
#define E2 11

static float FitP(float x, float PP[])	/* compute fit value */
{
	float F, bin, E, C;            
	int i;

	if(nP == E2) C = P1[E2];
	else C = PP[E2];
	bin = (x+0.5)*Scale + 16;
	E = PP[E0]+bin*PP[E1]*(1.F+bin*C);
	
	for(i=0, F=0.F; i<7; i++) {
		F += gaus(E, (i+1)*PP[T], PP[i], PP[SIG]);
	}
	return F;
}


static float dFitP(int iX, int iP, float PP[])
{
	int i;
	float dF, bin, E, C;
	
	if(nP == E2) C = P1[E2];
	else C = PP[E2];
	bin = (iX+0.5)*Scale + 16;
	E = PP[E0]+bin*PP[E1]*(1.F+bin*C);
	
	if(iP < 7) 	//amplitude
		return gaus(E, (iP+1)*PP[T], 1.F, PP[SIG]);
	
	else if(iP == SIG) // sigma
		for(i=0, dF=0.F; i<7; i++) 
			dF += gaus(E, (i+1)*PP[T], PP[i], PP[SIG])
				* (E-(i+1)*PP[T]) * (E-(i+1)*PP[T]) / 
				(PP[SIG]*PP[SIG]*PP[SIG]);

	else if(iP == T)	// T
		for(i=0, dF=0.F; i<7; i++) 
			dF += gaus(E, (i+1)*PP[T], PP[i], PP[SIG])
				* (E-(i+1)*PP[T]) * (i+1) / (PP[SIG]*PP[SIG]);
		
	else if(iP == E0) 	// energy offset
		for(i=0, dF=0.F; i<7; i++) 
			dF -= gaus(E, (i+1)*PP[T], PP[i], PP[SIG])
				* (E-(i+1)*PP[T]) / (PP[SIG]*PP[SIG]);

	else if(iP == E1) 	// energy linear term
		for(i=0, dF=0.F; i<7; i++) 
			dF -= gaus(E, (i+1)*PP[T], PP[i], PP[SIG])
				* (E-(i+1)*PP[T]) * bin * (1.F+bin*C) / (PP[SIG]*PP[SIG]);

	else if(iP == E2) 	//energy quadratic term
		for(i=0, dF=0.F; i<7; i++) 
			dF -= gaus(E, (i+1)*PP[T], PP[i], PP[SIG])
				* (E-(i+1)*PP[T]) * PP[E1] * bin * bin / (PP[SIG]*PP[SIG]);

	return dF;
}

static void FitSTEP()
{
	int i, j, k, RATES, SELECT, SINGLE, DETECTOR, CURVATURE, j0;
	float Chisq, Sigma;
	float X[256], YMax;
	int iYMax;
	
	nP = 12;
	nF = 1;
	P0[0] = 0;
	func[0] = FitP;
	dfunc[0] = dFitP;
	
	GetCtrlVal(STEFitPPanel, STEFITP_Select, &SELECT);
	j0 = SELECT?(NumEnergy0*4):0;
	GetCtrlVal(PlotSTESpecPanel, STESPEC_SingleDet, &SINGLE);
	GetCtrlVal(STEFitPPanel, STEFITP_Detector, &DETECTOR);
	if(!SINGLE) j0 += DETECTOR*NumEnergy0;
	GetCtrlVal(STEFitPPanel, STEFITP_FitCurvature, &CURVATURE);
	
	if(!CURVATURE) nP--;
	
	Scale = 256/NumEnergy0;
	if(SINGLE) Scale /= 4;

	for(i=16/Scale,nY=0; i<255/Scale; i++,nY++) {
		Y[nY]=Spectra[i+j0];
		Sig[nY]=sqrt((double)Y[nY])+1.F;
	}
	
	for(i=nY-(256/(Scale*8))-5, YMax=0.F, iYMax=nY; i<nY; i++)
		if(Y[i] > YMax) YMax = Y[iYMax = i];
	
	for(i=0; i<7; i++) {
		P1[i] = 50.F*AccTime*Scale;		// amplitude
	}
	P1[SIG] = 3.F/Scale;	//sigma
	P1[E0] = 0.F;
	P1[E1] = 0.39F;
	P1[T] = P1[E1]*(iYMax*Scale+16)/7.F;	// pulser gain
	if(CURVATURE) P1[E2]=0.F;
	else GetCtrlVal(STEFitPPanel, STEFITP_E2, &P1[E2]);

	Result = "OK";
	Chisq = lsq();
	if(P1[SIG] < 0.F) P1[SIG] = -P1[SIG];

	SetCtrlVal(STEFitPPanel, STEFITP_ChiSquared, Chisq);
	ResetTextBox(STEFitPPanel, STEFITP_Msg, Result);
	SetCtrlVal(STEFitPPanel, STEFITP_A1, P1[0]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	SetCtrlVal(STEFitPPanel, STEFITP_A2, P1[1]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	SetCtrlVal(STEFitPPanel, STEFITP_A3, P1[2]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	SetCtrlVal(STEFitPPanel, STEFITP_A4, P1[3]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	SetCtrlVal(STEFitPPanel, STEFITP_A5, P1[4]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	SetCtrlVal(STEFitPPanel, STEFITP_A6, P1[5]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	SetCtrlVal(STEFitPPanel, STEFITP_A7, P1[6]*P1[SIG]*G/(AccTime*P1[E1]*Scale));
	Sigma = P1[SIG]*P1[SIG]-P1[E1]*P1[E1]/12.;	// deconvole bin width
	if(Sigma < 0.) Sigma = 0.;
	Sigma = sqrt((double)Sigma);
	SetCtrlVal(STEFitPPanel, STEFITP_FWHM, Sigma*SIG2FWHM);
	SetCtrlVal(STEFitPPanel, STEFITP_E0, P1[E0]);
	SetCtrlVal(STEFitPPanel, STEFITP_E1, P1[E1]);
	SetCtrlVal(STEFitPPanel, STEFITP_E2, P1[E2]);
	SetCtrlVal(STEFitPPanel, STEFITP_T, P1[T]);
	
	GetCtrlVal(PlotSTESpecPanel, STESPEC_RATES, &RATES);

	for(i=0; i<nY; i++) {
		Y[i] = FitP((float)i, P1);
		if(RATES) Y[i] /= AccTime;
		X[i] = (float)(i+16.F/Scale);
	}
	PlotXY (PlotSTESpecPanel, STESPEC_GRAPH, X, Y, nY, VAL_FLOAT, VAL_FLOAT, VAL_SCATTER,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
    
}


int CVICALLBACK STEFit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			if(STEFitPanel == -1) STEFitPanel = LoadPanel(PlotSTESpecPanel, "DECOM.uir", STEFIT);
			DisplayPanel(STEFitPanel);
			FitSTE();
			break;
		}
	return 0;
}

int CVICALLBACK STEFitP (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			if(STEFitPPanel == -1) STEFitPPanel = LoadPanel(0, "DECOM.uir", STEFITP);
			DisplayPanel(STEFitPPanel);
			FitSTEP();
			break;
		}
	return 0;
}

int CVICALLBACK HideSTEFit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
		}
	return 0;
}
