// SWEA
//   Display SWEA rates

#include <stdio.h>
#include <userint.h>
#include <ansi_c.h>
#include "DECOM.h"

extern int PlotSWEARatesPanel, MainPanel;
extern unsigned char Data[];
int SwapWord(unsigned char *Data, int X); 
void SwapTelemetryBytes(char *Data);
int GetByte(int i);
unsigned int GetWord(int i);
long GetLong3412(int i);
char *GetTime1(unsigned int UTCs, char *dtime);
char *GetTime2(unsigned int UTCs, char *dtime);
char *GetTimeX(unsigned int UTCs, char *dtime);
int GetData(void);
static int Colors[16] = {
	VAL_BLACK, 0x803000L, VAL_RED, 0xFF8000L, 0xFFC000L, VAL_YELLOW, 0xC0FF00L,
	VAL_GREEN, 0x00FFFFL, VAL_BLUE, 0x8000FF, 0xFF00FFL, VAL_DK_MAGENTA,
	VAL_DK_GRAY, VAL_GRAY, VAL_WHITE
};

#define SIZE 1000000

int CVICALLBACK PlotSWEARates (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	extern float Samples[24][SIZE];
	extern long UTC[SIZE];
	int NS, Interval, i, j, AS, LOG, Dat, SEL, Col, Energy0, D0, A0, Delta;
	long UTC0;
	double Min, Max;
	float Val;
	char Str[100];
	static int Sel[16] = {
		SWEARATES_DET0, SWEARATES_DET1, SWEARATES_DET2, SWEARATES_DET3,
		SWEARATES_DET4, SWEARATES_DET5, SWEARATES_DET6, SWEARATES_DET7,
		SWEARATES_DET8, SWEARATES_DET9, SWEARATES_DET10, SWEARATES_DET11,
		SWEARATES_DET12, SWEARATES_DET13, SWEARATES_DET14, SWEARATES_DET15
	};
	static int SelE[16] = {
		SWEARATES_E0, SWEARATES_E1, SWEARATES_E2, SWEARATES_E3,
		SWEARATES_E4, SWEARATES_E5, SWEARATES_E6, SWEARATES_E7,
		SWEARATES_E8, SWEARATES_E9, SWEARATES_E10, SWEARATES_E11,
		SWEARATES_E12, SWEARATES_E13, SWEARATES_E14, SWEARATES_E15
	};
	static int SelD[6] = {
		SWEARATES_D0, SWEARATES_D1, SWEARATES_D2, SWEARATES_D3,
		SWEARATES_D4, SWEARATES_D5
	};
	
	static int SELE[16], SELD[6];

	switch (event) {
		case EVENT_COMMIT:
			DisplayPanel ( PlotSWEARatesPanel );
			DeleteGraphPlot (PlotSWEARatesPanel, SWEARATES_GRAPH, -1,
                 VAL_IMMEDIATE_DRAW);
            ClearAxisItems (PlotSWEARatesPanel, SWEARATES_GRAPH, VAL_XAXIS );
			SetWaitCursor(1);
			for(i=0; i<16; i++) GetCtrlVal(PlotSWEARatesPanel, SelE[i], &SELE[i]);
			for(i=0; i<6; i++) GetCtrlVal(PlotSWEARatesPanel, SelD[i], &SELD[i]);
			NS=-1;
			while(GetData() && (NS<SIZE)) if((SwapWord(Data, 0)&0x7FF) == 0x210) {
				SwapTelemetryBytes(Data);
				UTC0 = GetLong3412(32);
				Interval = GetByte(26+10);
				Energy0 = GetByte(26+12);
				switch (Energy0) {
					case 0 :
						UTC[++NS] = UTC0;
						for(i=0; i<16; i++) Samples[i][NS] = 0.;
						D0 = A0 = Delta = 0;
						break;
					case 3 :
						Delta = 16;
						D0 = 1;
						A0 = 8;
						break;
					case 6 :
						Delta = 32;
						D0 = 2;
						A0 = 8;
						break;
					case 9 :
						Delta = 48;
						D0 = 3;
						A0 = 8;
						break;
					case 12 :
						Delta = 64;
						D0 = 4;
						A0 = 8;
						break;
					default :
						Energy0 = -1;
						break;
				}
				if((Energy0 >= 0) && (NS >= 0)) for(i=0; i<256; ) {	// 256 samples per packet
					if(SELE[Energy0]) {
						if(SELD[D0]) {
							Dat = GetByte(26+15+i);
							Val = ((float)((Dat < 32)? Dat: (16+(Dat&0xf))<<(Dat/16 -1)))/(.0174/2.);
							if((D0 == 0) || (D0 == 5)) {
								Samples[A0++][NS] += Val/2;
								Samples[A0++][NS] += Val/2;
							}
							else Samples[A0++][NS] += Val;
							i++;
						}
						else {
							if((D0 == 0) || (D0 == 5)) i+=8;
							else i+= 16-A0;
							A0 = 0;
							D0++;
						}
						if(A0 > 15) {
							A0 = 0;
							D0++;
						}
						if(D0 > 5) {
							D0 = Delta = 0;
							Energy0++;
						}
					}
					else {
						i += 80-Delta; 
						D0 = A0 = Delta = 0;
						Energy0++;
					}
				}
			}

			GetCtrlVal(PlotSWEARatesPanel, SWEARATES_AutoScale, &AS);
			if(AS) {
				Min = 1.E6;
				Max = 1.;
				for(j=0; j<16; j++) {
					GetCtrlVal(PlotSWEARatesPanel, Sel[j], &SEL);
					if(SEL) for(i=0; i<NS; i++) {
						if (Samples[j][i] < Min) Min = Samples[j][i];
						if (Samples[j][i] > Max) Max = Samples[j][i];
					}
					if(Min < 1.) Min = 1.;
				}
				SetCtrlVal(PlotSWEARatesPanel, SWEARATES_MIN, Min);
				SetCtrlVal(PlotSWEARatesPanel, SWEARATES_MAX, Max);
			}
			else {
				GetCtrlVal(PlotSWEARatesPanel, SWEARATES_MIN, &Min);
				GetCtrlVal(PlotSWEARatesPanel, SWEARATES_MAX, &Max);
			}
			GetCtrlVal(PlotSWEARatesPanel, SWEARATES_Log, &LOG); 
			SetCtrlAttribute (PlotSWEARatesPanel, SWEARATES_GRAPH,
				ATTR_YMAP_MODE, LOG?VAL_LOG:VAL_LINEAR);			
			SetAxisScalingMode (PlotSWEARatesPanel, SWEARATES_GRAPH,
                    VAL_LEFT_YAXIS, VAL_MANUAL, Min, Max);
                    
			for(i=Col=0; i<16; i++, Col++) {
				GetCtrlVal(PlotSWEARatesPanel, Sel[i], &SEL);
       			SetCtrlAttribute(PlotSWEARatesPanel, Sel[i], ATTR_ON_COLOR,
       				SEL?Colors[Col]:VAL_WHITE);
				if(SEL)
				PlotXY (PlotSWEARatesPanel, SWEARATES_GRAPH, UTC, Samples[i], NS,
       				VAL_UNSIGNED_INTEGER, VAL_FLOAT, VAL_THIN_LINE, VAL_NO_POINT,
       				VAL_SOLID, 1, Colors[Col]);
			}
			InsertAxisItem (PlotSWEARatesPanel, SWEARATES_GRAPH, VAL_XAXIS, -1,
				GetTime1(UTC[0], Str), UTC[0]);
			InsertAxisItem (PlotSWEARatesPanel, SWEARATES_GRAPH, VAL_XAXIS, -1,
				GetTime1(UTC[NS-1], Str), UTC[NS-1]);
			
			SetWaitCursor(0);
			break;
		}

	return 0;
}


