// erftest.c
#include <utility.h>
#include <userint.h>
#include <ansi_c.h>
#include "erftest.h"

int MainPanel, ResultPanel;

int main (int Argc, char *argv[])
{
	void erfplot(void);

	if (InitCVIRTE (0, argv, 0) == 0) return -1;
	
	MainPanel = LoadPanel(0, "erftest.uir", PANEL);
	DisplayPanel ( MainPanel );

	erfplot();
	
	RunUserInterface ();
	
	return 0;
}

float Y[100], Sig[100], P1[10];
int nY, nP, nF, P0[10]; 
float (*func[10])(float x, float P[]), (*dfunc[10])(int iX, int iP, float P[]);
char *Result;

static float Fit(float x, float PP[])	/* compute fit value */
{
	float X;

	X = (x-PP[2])/PP[1];
	return 0.3989*PP[0]*exp(-X*X/2);
}

static float dFit(int iX, int iP, float PP[])
{
	float X;
	
	
	X = (iX-PP[2])/PP[1];

	if(iP == 0) {	//amplitude
		return 0.3989*exp(-X*X/2);
	}
	
	if(iP == 1) {	// sigma
		return 0.3989*PP[0]*exp(-X*X/2)*X*X/PP[1];
	}
	
	if(iP == 2) {	// energy offset
		return -0.3989*PP[0]*exp(-X*X/2)*X/PP[1];
	}
	return 0.F;
}

void err_out(char *Error)
{
	Result = Error;
}

void FitG(float F[], int N)
{
	int i;
	float Chisq, lsq(void);
	
	P0[0] = 0;
	func[0] = Fit;
	dfunc[0] = dFit;
	nP = 3;
	nF = 1;
	
	nY = N;
	for(i=0; i<nY; i++) {
		Y[i] = F[i];
		Sig[i] = 1.;
	}
	P1[0] = 1.;
	P1[1] = 1.;
	P1[2] = (float)nY/2.;
	Result = "OK";
	
	Chisq = lsq();
	
		
}

void erfplot(void)
{
	float X[100], E[100], G[100], dx;
	int i, N;
    double erf(double x); 
	
	N=30;
	dx = 1.;
	for(i=0; i<N; i++) {
		X[i] = (i-N/2)*dx;
		E[i] = (erf((double)(X[i]+dx/2)/1.414)-erf((double)(X[i]-dx/2)/1.414))/2.;
		G[i] = 0.3989*exp(-X[i]*X[i]/2)*dx;
	}

	FitG(E, N);
	SetCtrlVal(MainPanel, PANEL_EAmplitude, P1[0]*P1[1]);
	SetCtrlVal(MainPanel, PANEL_ESigma, P1[1]*dx);
	SetCtrlVal(MainPanel, PANEL_EOffset, P1[2]);
	
	FitG(G, N);
	SetCtrlVal(MainPanel, PANEL_GAmplitude, P1[0]*P1[1]);
	SetCtrlVal(MainPanel, PANEL_GSigma, P1[1]*dx);
	SetCtrlVal(MainPanel, PANEL_GOffset, P1[2]);
	
	SetAxisScalingMode (MainPanel, PANEL_GRAPH,
                    VAL_LEFT_YAXIS, VAL_MANUAL, .0001F, 10.F);
	PlotXY (MainPanel, PANEL_GRAPH, X, E, N, VAL_FLOAT, VAL_FLOAT, VAL_CONNECTED_POINTS,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
	PlotXY (MainPanel, PANEL_GRAPH, X, G, N, VAL_FLOAT, VAL_FLOAT, VAL_CONNECTED_POINTS,
       				VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_BLUE);
}

int CVICALLBACK Panel (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_CLOSE:
			exit(0);

			break;
		}
	return 0;
}
