REM Build_Ambient_PR0PT1_CmdSeq26.cmd
REM 
REM PR0  - Restore HV for all instruments at ambient levels
REM
REM
REM  12-04-05 version
REM
REM  This cmd file builds the command sequence (26) used to
REM  actually restore the HV for instruments in ambient.
REM  
REM
REM  Builds GSE command sequence CmdSeq26_Ambient_PR0PT1_Load
REM  which must be run before checking before enabling SCSTATUS checks
REM  and PAC Discharge 
REM
REM   Should be started using 
REM     /PLASTICStartCseqIfNotRunning
REM   to keep from aborting an executing version and restarting it.
REM
REM  Constraints checked:
REM     This command sequence will not call the guts of the RESTORE HV if:
REM       1.Any of IDPU or PLASTIC power,Thruster/Offpointing bits
REM         are set in SC Status word
REM       2.If SCFLAGS are set after we get into the logic
REM          after (HV restore) we won't enable entrance system and
REM          will leave in MODE 0. ON CLEAR OF BITS PR1 SHOULD COMPLETE
REM          THE RECOVERY
REM
REM  Constraints checked by the PLASTIC Constarint checking SW
REM     1. PR0 AND PS1 UP - ABORT BOTH START PS2
REM     2. ABORT PRO IF PS2 OR PS3 CALLED  
REM       
REM
REM     Note the sequence is split into 2 sequences
REM     because of the 128 byte limitation
REM     Second half is in cmdseq 23
REM
StartSequence RAM 26 CmdSeq26_Ambient_PR0PT1_Load.cmd


; check sc state flags to see if clear and are clear afte 2+ minutes


/PLASTICTestSCStateFALSE 0x39 0x00 26;Check bits 0+3 (IDPU+PLASTIC) power                                    ;down bits
                                     ;and 4+5 (Thruster+Offpointing) bits
                                ;Abort self (seq 25) if not all 0



/DelayCmdSeq 130;       ;check again to see if still clear
/PLASTICTestSCStateFALSE 0x39 0x00 26


/PLASTICSetMode0                ;Put in Mode 0
/DelayCmdSeq 62;                ;Give it time to get there
;
/PLASTICDigHkpDisable   ;disable dig hkp collection
/PLASTICAbortRamp       ;abort any active HV ramp (just in case)
/PLASTICClearAndDisableTOFSupplies      ;Clear pac,ssd,mcp csp an fsp
                                        ; and limits+disable
/DelayCmdSeq 2          ;Give some time
REM
REM  Modified to inform FSW that PAC Settings need to be read
REM


/PLASTIC_REFRESH_DISCHARGE_VARS ; use next DAC readback vars for
                                ;baseline HV control varaiables
/PLASTICDigHkpGet1Sample


;
;               Set PAC 5kV - Mark's seqeunce
/PLASTICSetPACCoarseLimit 0x0020        ;Set PAC limit to 20h 
/PLASTICEnablePAC                       ;Enable PAC


/PLASTICRampUpPAC 0x0020 0x00ff 0xff03 ; PAC coarse set point = 20
                                ; Ramp up start at 0, end ffH
                                ; step 255 count/3sec
                                ;limit should keep to 20h
/DelayCmdSeq 6
;
;       Set MCP 100V - Mark's seqeuence 
;
/PLASTICSetMCPCoarseLimit 0x000f  ;MCP limit =0f (0f shifted 3 bits left)
/PLASTICEnableMCP                 ;Enable MCP
/PLASTICRampUpMCP 0x000f 0x00ff 0xff03;MCP Coase set point = 0f
                                ;Ramp up start at 0, End = FF
                                ; step 255 count/3 sec
                                ;Limit should keep to 0fh
/DelayCmdSeq 6
/PLASTICDigHkpGet1Sample
/DelayCmdSeq 6


/PLASTICStartCSeqIfNotRunning 23;       Start 2nd half
EndSequence


REM Build_Ambient_PR0PT2_CmdSeq23.cmd
REM 
REM PR0  - Restore HV for all instruments at ambient levels
REM  PART2
REM
REM 12-04-2005 - SSd ASIC Set up
REM
REM  This cmd file builds the command sequence (23) used to
REM  actually restore the HV for instruments in ambient second half.
REM  
REM
REM  Builds GSE command sequence CmdSeq23_Ambient_PR0PT1_Load
REM  which must be run before checking before enabling SCSTATUS checks
REM  and PAC Discharge 
REM
REM   Should be started using 
REM     /PLASTICStartCseqIfNotRunning
REM   to keep from aborting an executing version and restarting it.
REM
REM  Constraints checked:
REM     This command sequence will not call the guts of the RESTORE HV if:
REM       1.Any of IDPU or PLASTIC power,Thruster/Offpointing bits
REM         are set in SC Status word
REM       2.If SCFLAGS are set after we get into the logic
REM          after (HV restore) we won't enable entrance system and
REM          will leave in MODE 0. ON CLEAR OF BITS PR1 SHOULD COMPLETE
REM          THE RECOVERY
REM
REM  Constraints checked by the PLASTIC Constarint checking SW
REM     1. PR0 AND PS1 UP - ABORT BOTH START PS2
REM     2. ABORT PRO IF PS2 OR PS3 CALLED  
REM       
REM
REM     Note the sequence is split into 2 sequences
REM     because of the 128 byte limitation
REM     Second half is in cmdseq 23
REM
StartSequence RAM 23 CmdSeq23_Ambient_PR0PT2_Load.cmd
;
;       SSD up to 100V note in 2 steps
;                       
/PLASTICEnableSSD                        ;Enable SSD


/PLASTICRampUpSSD 0x00ff 0x0070 0x0201;SSD coarse set point = ff
                                 ;Ramp up starting at 0, end 70H
                                 ; Step 2 counts/sec
/DelayCmdSeq 60                  ; Time to finish
/PLASTICRampUpSSD 0x00ff 0x70de 0x6e01 ;Step to deH in 1 step
/DelayCmdSeq 4                           ; Should be enuf
;<<<<<<<<<<<<<<<<<
/PLASTICIssueCommandMacro 10    ;ssd asic set-up
/PLASTICDigHkpGet1Sample   ; can put them back to back because macros suspend dighkp
/DelayCmdSeq 10
;<<<<<<<<<<<<<<<<<


;
;       Here if Offpointing,Thruster, IDPU or PLASTIC poweerdown bits
;       set don't continue
;
;/PLASTICTestSCStateFALSE 0x39 0x00 23  ;abort self if nonzero
/PLASTICEnableEntranceSystem            ;Enable Entrance system macro 1
/DelayCmdSeq 3


/PLASTICSetMode3                ;Mode 3


/PLASTICDigHkpEnable            ;enable dig hkp


/delaycmdseq 61
;
EndSequence
